package de.neusta.imonitor.correlation.configs;

/*
 * #%L
 * correlation
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

@Component
@Scope(value = "singleton")
public class CorrelationConfig extends Config {

	private static final int DEFAULT_INDIVIDUAL_GARBAGE_COLLECTOR_SIZE = 10000;

	private final String configFile = "../configs/correlation.properties";

	private static Logger LOGGER = LoggerFactory.getLogger(CorrelationConfig.class);

	private CorrelationConfig() {
		if (!this.loadConfig(configFile)) {
			LOGGER.error("Could not load correlation config file: " + configFile);
		}
	}

	public String getRuleFile() {
		return getProperty("rulefile");
	}

	public int getCores() {
		String coresString = getProperty("cores");
		if (!StringUtils.isEmpty(coresString)) {
			try {
				int parsedInt = Integer.parseInt(coresString);
				return parsedInt;
			} catch (NumberFormatException ex) {
				LOGGER.warn("Could not parse number of cores from the config file. Using single core.", ex);
				return 1;
			}
		} else {
			LOGGER.warn("No cores specified in config file. Using single core.");
			return 1;
		}
	}
	
	public int getIndividualGarbageCollectorSize() {
		String coresString = getProperty("individualGarbageCollectorSize");
		if (!StringUtils.isEmpty(coresString)) {
			try {
				int parsedInt = Integer.parseInt(coresString);
				return parsedInt;
			} catch (NumberFormatException ex) {
				LOGGER.warn("Could not parse number of individualGarbageCollectorSize from the config file. Using default value '"+DEFAULT_INDIVIDUAL_GARBAGE_COLLECTOR_SIZE+"'.", ex);
				return DEFAULT_INDIVIDUAL_GARBAGE_COLLECTOR_SIZE;
			}
		} else {
			LOGGER.warn("No individualGarbageCollectorSize specified in config file. Using default value '"+DEFAULT_INDIVIDUAL_GARBAGE_COLLECTOR_SIZE+"'.");
			return DEFAULT_INDIVIDUAL_GARBAGE_COLLECTOR_SIZE;
		}
	}

	public boolean check() {
		boolean success = true;
		String ruleFile = getRuleFile();
		if (ruleFile == null || ruleFile.isEmpty()) {
			LOGGER.error("The rule file has not been specified in the correlation properties file.");
			success = false;
		}
		return success;
	}

}
