package de.neusta.imonitor.correlation.configs;

/*
 * #%L
 * correlation
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import javax.annotation.PostConstruct;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

@Service
public class InputConfig extends Config{

	private final String configFile = "../configs/input.properties";
	
	private static final Logger LOGGER = LoggerFactory.getLogger(InputConfig.class);
	
	@PostConstruct
	private void init(){
		if (!this.loadConfig(configFile)){
			LOGGER.error("Could not load input config file: " + configFile);
		}
	}
	
	public InputMethod getInputMethod(){
		InputMethod result = InputMethod.Unknown;
		String inputType = getProperty("inputMethod");
		if (inputType != null)
		{
			if (inputType.toLowerCase().equals("pipe")){
				result = InputMethod.Pipe;
			} else if (inputType.toLowerCase().equals("file")){
				result = InputMethod.File;
			}
		}
		return result;
	}

	public String getPipeName() {
		return getProperty("pipeName");
	}
	
	public String getFileName() {
		return getProperty("fileName");
	}
	
	public String getDefaultNamespace() {
		return getProperty("defaultNamespace");
	}

	public boolean check(){
		boolean success = true;
		String defaultNamespace = getDefaultNamespace(); 
		if (defaultNamespace == null || defaultNamespace.isEmpty()){
			LOGGER.warn("The default namespace has not been specified in the input properties.");
		}
		InputMethod inputMethod = getInputMethod();
		if (inputMethod == InputMethod.Unknown){
			LOGGER.error("An unknown input method has been specified in the input properties.");
			success = false;
		}
		if (inputMethod == InputMethod.Pipe || inputMethod == InputMethod.File){
			InputFormat inputFormat = getInputFormat();
			if (inputFormat == InputFormat.Unknown)
			{
				LOGGER.error("An unknown input format has been specified in the input properties.");
				success = false;
			}
		}
		return success;
	}

	public InputFormat getInputFormat() {
		InputFormat result = InputFormat.Unknown;
		String inputFormat = getProperty("inputFormat");
		if (inputFormat != null)
		{
			if (inputFormat.toLowerCase().equals("csv")){
				result = InputFormat.CSV;
			} else if (inputFormat.toLowerCase().equals("json")){
				result = InputFormat.JSON;
			}
		}
		return result;
	}

}
