package de.neusta.imonitor.correlation.input;

/*
 * #%L
 * correlation
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import java.io.IOException;
import java.io.StringReader;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import au.com.bytecode.opencsv.CSVReader;
import de.neusta.imonitor.correlation.configs.InputConfig;
import de.neusta.imonitor.correlation.model.VariableNode;
import de.neusta.imonitor.correlation.processors.CorrelationEventProcessor;
import de.neusta.imonitor.correlation.services.DefaultVariablesService;
import de.neusta.imonitor.correlation.services.ValueMappingService;
import de.neusta.imonitor.correlation.utils.LogUtils;
import de.neusta.imonitor.ontology.VariableList;

@Service
public class CSVProcessor implements ILineProcessor {

	private static Logger LOGGER = LoggerFactory.getLogger(CSVProcessor.class);

	@Autowired
	private CorrelationEventProcessor correlationEventProcessor;

	@Autowired
	private InputConfig inputConfig;

	@Autowired
	private DefaultVariablesService defaultVariablesService;
	
	@Autowired
	private ValueMappingService valueMappingService;
	
	/**
	 * Processes a single line of input as csv.
	 * 
	 * @param line
	 */
	@Deprecated
	public void processLine(String line) {
		try {
			// create input and call callback (i.e., the eventprocessor)
			CSVReader csvReader = new CSVReader(new StringReader(line), ',',
					'\"');
			try {
				String[] tokens;
				try {
					// parse input line into tokens (CSV)
					tokens = csvReader.readNext();
				} catch (IOException e) {
					LOGGER.error(String.format(
							"Exception while reading line '%s'.", line));
					return;
				}
				boolean validTokens = checkValidTokens(line, tokens);
				if (validTokens) {
					VariableList variableList = getVariableValuePairsFromTokens(tokens);
					if (variableList.getVariables().size() > 0) {
						valueMappingService.mapVariables(variableList);
						this.correlationEventProcessor.process(variableList, new VariableNode());
					}
				}
			} finally {
				csvReader.close();
			}
		} catch (Exception ex) {
			LogUtils.logException(LOGGER, ex);
		}
	}

	private VariableList getVariableValuePairsFromTokens(String[] tokens) {
		VariableList variableList = new VariableList();
		// get variable / value pairs from tokens
		try {
			for (int i = 0; i < tokens.length / 2; i++) {
				String variableName = tokens[i * 2];
				String value = tokens[i * 2 + 1];

				if (!value.contains(":")) {
					value = inputConfig.getDefaultNamespace() + ":" + value;
				}

				variableList.createVariable(variableName, value);
			}
		} catch (Exception e) {
			LOGGER.error(String.format(
					"Exception while creating variable list: %s", e));
		}
		defaultVariablesService.addDefaultVariables(variableList);
		return variableList;
	}

	/***
	 * Checks if the number of tokens are valid (an even number of tokens).
	 * 
	 * @param line
	 * @param tokens
	 * @return
	 */
	private boolean checkValidTokens(String line, String[] tokens) {
		if (tokens == null) {
			LOGGER.warn(String.format("Could not parse tokens from line '%s'.",
					line));
			return false;
		}
		if (tokens.length % 2 != 0) {
			LOGGER.warn(String
					.format("The number of arguments must be even. Omitting line '%s'.",
							line));
			return false;
		}
		return true;
	}
}
