package de.neusta.imonitor.correlation.input;

/*
 * #%L
 * correlation
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;

import javax.annotation.PostConstruct;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.neusta.imonitor.correlation.configs.InputConfig;
import de.neusta.imonitor.correlation.processors.CorrelationEventProcessor;
import de.neusta.imonitor.correlation.threadpool.CorrelationThreadPool;
import de.neusta.imonitor.correlation.utils.LogUtils;

@Service
public class FileProcessor implements Runnable, IInputProcessor {

	private static Logger LOGGER = LoggerFactory.getLogger(FileProcessor.class);
	
	private Logger PERFORMANCE_LOGGER = LoggerFactory.getLogger("performanceLogger");

	private ILineProcessor lineProcessor;
	
	@Autowired
	private InputConfig inputConfig;
	
	@Autowired
	private CorrelationEventProcessor correlationEventProcessor;
	
	@Autowired
	private LineProcessorFactory lineProcessorFactory;
	
	@Autowired
	private CorrelationThreadPool correlationThreadPool;
	
	@PostConstruct
	public void init(){
		this.lineProcessor = lineProcessorFactory.createLineProcessor();
	}

	@Override
	public Thread process() {
		Thread thread = new Thread(this);
		thread.start();
		return thread;
	}

	@Override
	public void run() {
		File file = new File(inputConfig.getFileName());
		LOGGER.info("Starting to process input from file...");
		long start = System.currentTimeMillis();
		int lines = 0;
		try {
			RandomAccessFile raf;
			raf = new RandomAccessFile(file, "r");
			String line = null;
			do {
				line = raf.readLine();
				LOGGER.trace("Processing line with content: '" + line + "'");
				if (line != null) {
					this.lineProcessor.processLine(line);
					lines++;
				}
				LOGGER.trace("Processed line: " + lines);
			} while (line != null);
			LOGGER.info("Closing input from file.");
			raf.close();
			LOGGER.info("Waiting for correlation to complete...");
			correlationThreadPool.waitForTasks();
		} catch (FileNotFoundException e) {
			LOGGER.error("Could not open file: " + inputConfig.getFileName());
			LogUtils.logException(LOGGER, e);
		} catch (IOException e) {
			LOGGER.error("Error while reading from file." + inputConfig.getFileName());
			LogUtils.logException(LOGGER, e);
		} catch(Exception e){
			LOGGER.error("Exception while processing file: " + inputConfig.getFileName());
			LogUtils.logException(LOGGER, e);
		}
		long stop = System.currentTimeMillis();
		float duration = stop - start;
		LOGGER.info("Elapsed milliseconds: " + (stop - start));
		PERFORMANCE_LOGGER.info("Elapsed milliseconds for reading the file: " + duration);
		PERFORMANCE_LOGGER.info("Processed lines from file: " + lines);
		PERFORMANCE_LOGGER.info((lines / (duration / 1000)) + " lines per second");
	}

}
