package de.neusta.imonitor.correlation.input;

/*
 * #%L
 * correlation
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import java.util.Set;

import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.neusta.imonitor.correlation.configs.InputConfig;
import de.neusta.imonitor.correlation.model.VariableNode;
import de.neusta.imonitor.correlation.processors.CorrelationEventProcessor;
import de.neusta.imonitor.correlation.service.OntologyVariableService;
import de.neusta.imonitor.correlation.services.DefaultVariablesService;
import de.neusta.imonitor.correlation.services.ValueMappingService;
import de.neusta.imonitor.correlation.transformer.JsonToVariableGraphTransformer;
import de.neusta.imonitor.correlation.utils.LogUtils;
import de.neusta.imonitor.ontology.VariableList;

@Service
public class JSONProcessor implements ILineProcessor {

	private static Logger LOGGER = LoggerFactory.getLogger(JSONProcessor.class);

	@Autowired
	private InputConfig inputConfig;
	@Autowired
	private CorrelationEventProcessor correlationEventProcessor;
	@Autowired
	private DefaultVariablesService defaultVariablesService;
	@Autowired
	private ValueMappingService valueMappingService;
	@Autowired
	private JsonToVariableGraphTransformer jsonToVariableGraphTransformer;
	@Autowired
	private OntologyVariableService ontologyVariableService;

	
	private JSONParser jsonParser;


	public JSONProcessor() {
		this.jsonParser = new JSONParser();
	}

	@Override
	public void processLine(String line) {
		try {
			LOGGER.info("Parsing JSON from data: '" + line + "'.");

			JSONObject jsonObject = (JSONObject) this.jsonParser.parse(line);

			VariableList variableList = this.getVariableValuePairsFromData(jsonObject);
			VariableNode variableGraphRoot = jsonToVariableGraphTransformer.transform(jsonObject);
			valueMappingService.mapVariables(variableList);
			this.correlationEventProcessor.process(variableList, variableGraphRoot);
		} catch (Exception e) {
			LogUtils.logException(LOGGER, e);
		}
	}

	private VariableList getVariableValuePairsFromData(JSONObject jsonData) {
		VariableList variableList = new VariableList();
		// get variable / value pairs from tokens
		try {
			Set<?> keys = jsonData.keySet();
			for (Object key : keys) {
				if (isFlatListEntry(jsonData, key)) {
					String variableName = (String) key;
					String value = (String) jsonData.get(key);
					variableList.createVariable(variableName, value);
				}
			}
		} catch (Exception e) {
			LOGGER.error(String.format("Exception while creating variable list: %s", e));
		}
		defaultVariablesService.addDefaultVariables(variableList);
		return variableList;
	}

	private boolean isFlatListEntry(JSONObject jsonData, Object key) {
		return key instanceof String && jsonData.get(key) instanceof String;
	}

}
