package de.neusta.imonitor.correlation.input;

/*
 * #%L
 * correlation
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;

import javax.annotation.PostConstruct;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.neusta.imonitor.correlation.configs.InputConfig;

@Service
public class PipeProcessor implements Runnable, IInputProcessor {
	
	private static Logger LOGGER = LoggerFactory.getLogger(PipeProcessor.class);
	
	private final int MaxIdleTime = 3000;
	
	private ILineProcessor lineProcessor;
	
	private String pipeName;
	
	@Autowired
	private InputConfig inputConfig;

	@Autowired
	private LineProcessorFactory lineProcessorFactory;

	public void setPipeName(String pipeName){
		this.pipeName = pipeName;
	}
	
	public String getPipeName(){
		return this.pipeName;
	}
	
	@PostConstruct
	public void init() {
		this.pipeName = inputConfig.getPipeName();
		this.lineProcessor = lineProcessorFactory.createLineProcessor();
	}

	@Override
	public Thread process() {
		Thread thread = new Thread(this);
		thread.start();
		return thread;
	}

	@Override
	public void run() {
		File pipe = new File(this.pipeName);
		LOGGER.info("Starting to process input from pipe...");
		try {
		while(true){
	          RandomAccessFile raf;
				raf = new RandomAccessFile(pipe, "r");
	          String line = null;  
	          int idleTime = 0; 
	          while( idleTime < MaxIdleTime ){
	                line=raf.readLine();         
 	                if( line!=null ) {
 	                	this.lineProcessor.processLine(line);
 	                	idleTime=0;
	               }else{
	                       Thread.sleep(100); 
	                       idleTime+=100;  
	               }
	          }
	          LOGGER.info("Closing input from pipe.");
	          raf.close();
		}
		} catch (FileNotFoundException e) {
			LOGGER.error("Could not open pipe: " + pipeName);
		} catch (IOException e) {
			LOGGER.error("Error while reading from pipe. Broken pipe?: " + pipeName);
		} catch (InterruptedException e) {
			LOGGER.trace("The pipe processor thread has been aborted.");
		}
	}

}
