package de.neusta.imonitor.correlation.patternmatcher;

/*
 * #%L
 * correlation
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.neusta.imonitor.correlation.services.VariableReplacementService;
import de.neusta.imonitor.ontology.Ontology;
import de.neusta.imonitor.ontology.VariableList;

@Service
public class PatternMatcher {

	private static Logger LOGGER = LoggerFactory.getLogger(PatternMatcher.class);

	@Autowired
	private Ontology ontology;

	@Autowired
	private VariableReplacementService variableReplacementService;

	public boolean matchPattern(Pattern pattern, VariableList variableList) throws Exception {
		String query = ontology.getSparqlPrefixes() + variableReplacementService.replace(pattern.getQuery(), variableList);
		// pattern does not match if there are unbound variables
		if (variableReplacementService.containsVariable(query)) {
			return false;
		}
		try {
			// NOTE [ce]: required due to a bug in pellet resulting in NPEs.
			this.ontology.writeLock();
			boolean result = this.ontology.ask(query);
			LOGGER.info("Pattern checked: " + result + ", query: " + query);
			return result;
		} finally {
			this.ontology.releaseLock();
		}
	}
}
