package de.neusta.imonitor.correlation.processors;

/*
 * #%L
 * correlation
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;

import de.neusta.imonitor.correlation.model.VariableNode;
import de.neusta.imonitor.correlation.threadpool.CorrelationThreadPool;
import de.neusta.imonitor.ontology.VariableList;
import de.neusta.imonitor.ontology.exception.VariableAlreadyExistsException;

@Service
public class CorrelationEventProcessor {

	private static Logger LOGGER = LoggerFactory.getLogger(CorrelationEventProcessor.class);

	@Autowired
	private ApplicationContext applicationContext;
	@Autowired
	private CorrelationThreadPool correlationThreadPool;

	public void process(VariableList variableList, VariableNode variableGraphRoot) throws VariableAlreadyExistsException {
		try {
			CorrelationEventProcessorTask correlationTask = applicationContext.getBean(CorrelationEventProcessorTask.class);
			correlationTask.setVariableList(variableList);
			correlationTask.setVariableGraph(variableGraphRoot);
			correlationThreadPool.execute(correlationTask);
		} catch (Exception ex) {
			LOGGER.error("Unexpected exception while adding event correlation task to thread pool.", ex);
		}
	}
}
