package de.neusta.imonitor.correlation.processors;

/*
 * #%L
 * correlation
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import java.util.HashMap;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import de.neusta.imonitor.correlation.services.QueryFactoryService;
import de.neusta.imonitor.correlation.utils.ConversionUtils;
import de.neusta.imonitor.ontology.Ontology;
import de.neusta.imonitor.ontology.VariableList;
import de.neusta.imonitor.ruleparser.abstraction.Abstraction;
import de.neusta.imonitor.ruleparser.rules.Action;

@Component
public class UpdateProcessor {

	@Autowired
	private Ontology ontology;

	@Autowired
	private QueryFactoryService queryFactoryService;
	
	/**
	 * Performs all updates on the ontologies specified in the given action.
	 * 
	 * @param action
	 * @param variableList
	 * @param actionQueryVariables
	 * @param abstraction
	 * @throws Exception
	 */
	public void process(Action action, VariableList variableList,
			HashMap<String, List<String>> actionQueryVariables,
			Abstraction abstraction) throws Exception {

		List<String> updateQueries = action.getUpdateQueries();
		for (String updateQuery : updateQueries) {
			String query = queryFactoryService.createQuery(abstraction,
					updateQuery);
			String updatedQueryWithReplacements = ConversionUtils.replaceVariablesInString(variableList, actionQueryVariables, query);
			try
			{
				this.ontology.writeLock();
			ontology.update(this.ontology
					.getSparqlPrefixes() + updatedQueryWithReplacements);
			} finally {
				this.ontology.releaseLock();
			}
		}

	}

}
