package de.neusta.imonitor.correlation.ruleloader;

/*
 * #%L
 * correlation
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import java.io.StringReader;
import java.util.List;

import javax.transaction.Transactional;

import org.dom4j.Document;
import org.dom4j.io.SAXReader;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.neusta.imonitor.correlation.persistency.entities.RuleEntity;
import de.neusta.imonitor.correlation.persistency.repositories.RuleRepository;
import de.neusta.imonitor.ontology.Ontology;
import de.neusta.imonitor.ruleparser.exception.RuleValidationException;
import de.neusta.imonitor.ruleparser.rules.RuleParser;

@Service
public class RuleLoader {

	@Autowired
	private RuleRepository ruleRepository;

	private static final Logger LOGGER = LoggerFactory.getLogger(RuleLoader.class);

	@Autowired
	private RuleParser ruleParser;

	public void loadRuleFile(String uri, Ontology ontology) throws Exception {
		LOGGER.trace("Loading rules from file: " + uri);
		SAXReader reader = new SAXReader();
		Document document = reader.read(uri);
		this.ruleParser.parseDocument(document);
	}

	@Transactional
	public void loadRulesFromDatabase(Ontology ontology) throws Exception {
		LOGGER.trace("Loading rules from database.");
		this.ruleParser.clear();
		SAXReader reader = new SAXReader();
		List<RuleEntity> rules = ruleRepository.findAll();
		for (RuleEntity ruleEntity : rules) {
			LOGGER.trace("Loading rule '" + ruleEntity.getName() + "' with id '" + ruleEntity.getId() + "'.");
			String ruleXml = ruleEntity.getRule();
			Document document = reader.read(new StringReader(ruleXml));
			try {
				boolean successfullyLoadedRule = this.ruleParser.parseAndAddRule(document);
				if (!successfullyLoadedRule) {
					LOGGER.error("Could not load/parse rule '" + ruleEntity.getName() + "' with id '" + ruleEntity.getId() + "'.");
				}
			} catch (RuleValidationException ex) {
				LOGGER.error("Could validate rule '" + ruleEntity.getName() + "' with id '" + ruleEntity.getId() + "'.");
				throw ex;
			}
		}
		LOGGER.info("Loaded and parsed " + this.ruleParser.getRules().size() + " rules.");
	}
}
