package de.neusta.imonitor.correlation.services;

/*
 * #%L
 * correlation
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import java.io.IOException;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.neusta.imonitor.correlation.utils.LogUtils;
import de.neusta.imonitor.ontology.VariableList;


@Service
public class CommandService {
	private static final Logger LOGGER = LoggerFactory.getLogger(CommandService.class);
	
	@Autowired
	private MakroService makroService;
	
	public void processCommand(String command, List<String> explanations, List<String> recommendations, VariableList variableList) {
		if (command != null && !command.isEmpty()) {
			Runtime runtime = Runtime.getRuntime();
			try {
				command = makroService.substitute(command, explanations, recommendations, variableList);
				LOGGER.info(String.format("Executing command '%s'.", command));
				Process process = runtime.exec(command);
				int returnCode = process.waitFor();
				if (returnCode != 0) {
					LOGGER.warn("The execution of the command returned an error code.");
				}
			} catch (IOException | InterruptedException e) {
				LOGGER.error(String.format(
						"Exception while executing the command '%s'.", command));
				LogUtils.logException(LOGGER, e);
			}
		}
	}

}
