package de.neusta.imonitor.correlation.services;

/*
 * #%L
 * correlation
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import java.util.Date;
import java.util.UUID;

import org.apache.commons.lang3.time.DateUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.ibm.icu.util.Calendar;

import de.neusta.imonitor.correlation.transformer.DateToDatetimeTransformer;
import de.neusta.imonitor.ontology.VariableList;

@Service
public class DefaultVariablesService {

	private final static Logger LOGGER = LoggerFactory
			.getLogger(DefaultVariablesService.class);

	@Autowired
	private DateToDatetimeTransformer dateTransformer;

	public void addDefaultVariables(VariableList variableList) {
		try {
			addNow(variableList);
			addToday(variableList);
			addYesterday(variableList);
			add10MinuteAgo(variableList);
			add1HourAgo(variableList);
			variableList.createVariable("GUID1", UUID.randomUUID().toString());
			variableList.createVariable("GUID2", UUID.randomUUID().toString());
			variableList.createVariable("GUID3", UUID.randomUUID().toString());
			variableList.createVariable("DATA", UUID.randomUUID().toString());
		} catch (Exception e) {
			LOGGER.error("Could not add default date variables.", e);
		}
	}

	private void add10MinuteAgo(VariableList variableList)
			throws Exception {
		Calendar tenMinutesAgoCalendar = Calendar.getInstance();
		tenMinutesAgoCalendar.add(Calendar.MINUTE, -10);
		String tenMinutesAgo = dateTransformer.transform(tenMinutesAgoCalendar
				.getTime());
		variableList.createVariable("10MINUTESAGO", createLiteral(tenMinutesAgo));
	}

	private void add1HourAgo(VariableList variableList)
			throws Exception {
		Calendar tenMinutesAgoCalendar = Calendar.getInstance();
		tenMinutesAgoCalendar.add(Calendar.HOUR, -1);
		String oneHourAgo = dateTransformer.transform(tenMinutesAgoCalendar
				.getTime());
		variableList.createVariable("1HOURAGO", createLiteral(oneHourAgo));
	}

	private void addYesterday(VariableList variableList)
			throws Exception {
		Calendar yesterdayCalendar = Calendar.getInstance();
		yesterdayCalendar.add(Calendar.DAY_OF_MONTH, -1);
		Date yesterdayDate = DateUtils.truncate(yesterdayCalendar.getTime(),
				Calendar.DAY_OF_MONTH);
		String yesterday = dateTransformer.transform(yesterdayDate);
		variableList.createVariable("YESTERDAY", createLiteral(yesterday));
	}

	private void addToday(VariableList variableList)
			throws Exception {
		Date todayDate = DateUtils.truncate(Calendar.getInstance().getTime(), Calendar.DAY_OF_MONTH);
		String today = dateTransformer.transform(todayDate);
		variableList.createVariable("TODAY", createLiteral(today));
	}

	private void addNow(VariableList variableList)
			throws Exception {
		String now = dateTransformer.transform(Calendar.getInstance().getTime());
		variableList.createVariable("NOW", createLiteral(now));
	}

	private String createLiteral(String datetime) {
		return "\"" + datetime + "\"" + "^^xsd:dateTime";
	}

}
