package de.neusta.imonitor.correlation.services;

/*
 * #%L
 * correlation
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.transaction.Transactional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.neusta.imonitor.correlation.persistency.entities.ExplanationEntity;
import de.neusta.imonitor.correlation.persistency.entities.IncidentEntity;
import de.neusta.imonitor.correlation.persistency.entities.RecommendationEntity;
import de.neusta.imonitor.correlation.persistency.enums.IncidentStatus;
import de.neusta.imonitor.correlation.persistency.repositories.ExplanationRepository;
import de.neusta.imonitor.correlation.persistency.repositories.IncidentRepository;
import de.neusta.imonitor.correlation.persistency.repositories.RecommendationRepository;
import de.neusta.imonitor.ontology.Variable;
import de.neusta.imonitor.ontology.VariableList;
import de.neusta.imonitor.ruleparser.rules.Action;

@Service
public class IncidentService {
	
	@Autowired
	private IncidentRepository incidentRepository;
	
	@Autowired
	private RecommendationRepository recommendationRepository;
	
	@Autowired
	private ExplanationRepository explanationRepository;


	@Transactional
	public void writeIncidentToDatabase(Action action, VariableList variableList, List<String> explanations,
			List<String> recommendations) {
		Variable sourceIpVariable = variableList.getVariable("hostaddress");
		String sourceIp = "";
		if (sourceIpVariable != null) {
			sourceIp = sourceIpVariable.getValue();
		}
		IncidentEntity incidentEntity = new IncidentEntity();
		incidentEntity.setStatus(IncidentStatus.New);
		incidentEntity.setTimestamp(new Date());
		incidentEntity.setSourceIp(IpService.getIpWithoutNamespace(sourceIp));
		incidentEntity.setSourceIpOntology(sourceIp);
		incidentEntity.setRisk(action.getRisk());
		incidentEntity.setName(action.getIncidentName());
		setRecommendations(recommendations, incidentEntity);
		setExplanations(explanations, incidentEntity);
		incidentRepository.save(incidentEntity);
	}


	private void setExplanations(List<String> explanations,
			IncidentEntity incidentEntity) {
		List<ExplanationEntity> explanationEntities = new ArrayList<ExplanationEntity>();
		
		for (String explanation : explanations) {
			ExplanationEntity explanationEntity = new ExplanationEntity();
			explanationEntity.setIncident(incidentEntity);
			explanationEntity.setExplanation(explanation);
			explanationEntities.add(explanationEntity);
		}
		incidentEntity.setExplanations(explanationEntities);
	}


	private void setRecommendations(List<String> recommendations,
			IncidentEntity incidentEntity) {
		List<RecommendationEntity> recommendationEntities = new ArrayList<RecommendationEntity>();
		for (String recommendation : recommendations) {
			RecommendationEntity recommendationEntity = new RecommendationEntity();
			recommendationEntity.setIncident(incidentEntity);
			recommendationEntity.setRecommendation(recommendation);
			recommendationEntities.add(recommendationEntity);
		}
		incidentEntity.setRecommendations(recommendationEntities);
	}
}
