package de.neusta.imonitor.correlation.services;

/*
 * #%L
 * correlation
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import org.springframework.stereotype.Service;

import de.neusta.imonitor.correlation.model.UseAbstractionFunctionOccurence;
import de.neusta.imonitor.ruleparser.abstraction.Abstraction;
import de.neusta.imonitor.ruleparser.abstraction.AbstractionVariable;
import de.neusta.imonitor.ruleparser.abstraction.QueryHelper;

@Service
public class QueryFactoryService {

	private List<UseAbstractionFunctionOccurence> getAbstractionUsages(
			String query) throws Exception {
		int abstractionFunctionStartIndex;
		List<UseAbstractionFunctionOccurence> abstractionFunctionOccurences = new ArrayList<UseAbstractionFunctionOccurence>();
		int currentIndex = 0;
		do {
			// scan for abstraction functions
			String lowerCaseQuery = query.toLowerCase();
			abstractionFunctionStartIndex = lowerCaseQuery.indexOf(
					"useabstract(", currentIndex);
			if (abstractionFunctionStartIndex >= 0) {
				int abstractionFunctionEndIndex = lowerCaseQuery.indexOf(")",
						abstractionFunctionStartIndex);
				String argumentsString = query.substring(
						abstractionFunctionStartIndex + 12,
						abstractionFunctionEndIndex);
				StringTokenizer tokenizer = new StringTokenizer(
						argumentsString, ",");
				if (tokenizer.countTokens() != 2) {
					throw new Exception(
							"Invalid number of parameters for the abstract function in the rule.");
				}
				String concept = tokenizer.nextToken();
				String variableName = tokenizer.nextToken();

				UseAbstractionFunctionOccurence abstractionFunctionOccurence = new UseAbstractionFunctionOccurence();
				abstractionFunctionOccurence.abstractionFunctionStartIndex = abstractionFunctionStartIndex;
				abstractionFunctionOccurence.abstractionFunctionEndIndex = abstractionFunctionEndIndex;
				abstractionFunctionOccurence.concept = QueryHelper
						.removeQuotationMarks(concept.trim());
				abstractionFunctionOccurence.variableName = QueryHelper
						.removeQuotationMarks(variableName.trim());
				abstractionFunctionOccurences.add(abstractionFunctionOccurence);
				currentIndex = abstractionFunctionEndIndex;
			}
		} while (abstractionFunctionStartIndex >= 0);
		return abstractionFunctionOccurences;
	}

	/**
	 * Replaces the usage of "UseAbstraction" in a query.
	 * 
	 * @param abstraction
	 * @param abstractionUsages
	 * @param query
	 * @return
	 */
	private String replaceAbstractionUsages(Abstraction abstraction,
			List<UseAbstractionFunctionOccurence> abstractionUsages,
			String query) {
		String substitutedQuery = new String(query);
		int relativeIndex = 0;
		// replace all abstraction functions
		for (int i = 0; i < abstractionUsages.size(); i++) {
			UseAbstractionFunctionOccurence abstractionFunction = abstractionUsages
					.get(i);

			AbstractionVariable[] abstractionVariables = abstraction
					.getAbstractionVariables();
			String substitutionValue = null;
			for (AbstractionVariable abstractionVariable : abstractionVariables) {
				if (abstractionVariable.getVariableName().equalsIgnoreCase(
						abstractionFunction.variableName)) {
					substitutionValue = abstractionVariable.getVariableValue();
					break;
				}
			}
			if (substitutionValue == null) {
				substitutionValue = abstractionFunction.concept;
			}

			substitutedQuery = substitutedQuery.substring(0,
					abstractionFunction.abstractionFunctionStartIndex
							+ relativeIndex)
					+ substitutionValue
					+ substitutedQuery
							.substring(abstractionFunction.abstractionFunctionEndIndex
									+ 1 + relativeIndex);
			int abstractionFunctionLength = abstractionFunction.abstractionFunctionEndIndex
					- abstractionFunction.abstractionFunctionStartIndex + 1; // +1
																				// for
																				// the
																				// final
																				// bracket
			relativeIndex += substitutionValue.length()
					- abstractionFunctionLength;
		}
		return substitutedQuery;
	}

	/**
	 * Adjusts the given query string by replaceing usage operation (for using abstractions).
	 * @param abstraction
	 * @param query
	 * @return
	 * @throws Exception
	 */
	public String createQuery(Abstraction abstraction, String query) throws Exception {
		List<UseAbstractionFunctionOccurence> abstractionUsages = this.getAbstractionUsages(query);
		return this.replaceAbstractionUsages(abstraction, abstractionUsages, query);
	}

}
