package de.neusta.imonitor.correlation.services;

/*
 * #%L
 * correlation
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.transaction.Transactional;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.neusta.imonitor.correlation.persistency.entities.ValueMappingEntity;
import de.neusta.imonitor.correlation.persistency.repositories.ValueMappingRepository;
import de.neusta.imonitor.ontology.Variable;
import de.neusta.imonitor.ontology.VariableList;

@Service
public class ValueMappingService {
	
	private static final Logger LOGGER = LoggerFactory.getLogger(ValueMappingService.class);
	
	@Autowired
	private ValueMappingRepository valueMappingRepository;
	
	private Map<String, String> mappings;
	
	/**
	 * Maps the variable values according to the mapping values from the database.
	 * @param variableList
	 */
	@Transactional
	public void mapVariables(VariableList variableList) {
		if (mappings == null) {
			initMappings();
		}
		Collection<Variable> variables = variableList.getVariables();
		for (Variable variable : variables) {
			
			String mapping = mappings.get(variable.getName() + "-" + variable.getValue());
			if (mapping != null) {
				LOGGER.trace("Mapping '" + variable.getValue() + "' to '"+mapping + "'.");
				variable.setValue(mapping);
			}
		}
	}

	private void initMappings() {
		mappings = new HashMap<String, String>();
		List<ValueMappingEntity> mappingEntities = valueMappingRepository.findAll();
		for (ValueMappingEntity valueMappingEntity : mappingEntities) {
			String varName = valueMappingEntity.getVariableName();
			String value = valueMappingEntity.getValue();
			String mappedValue = valueMappingEntity.getMappedValue();
			mappings.put(varName + "-" + value, mappedValue);
		}
	}

}
