package de.neusta.imonitor.correlation.services;

/*
 * #%L
 * correlation
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import org.springframework.stereotype.Service;

import de.neusta.imonitor.ontology.Variable;
import de.neusta.imonitor.ontology.VariableList;

@Service
public class VariableReplacementService {
	
	/**
	 * Replaces all variable placeholders in a query string by the variable values.
	 * @param query
	 * @param variableList
	 * @return
	 */
	public String replace(String query,
			VariableList variableList) {
		String queryWithReplacements = query;
		for(Variable variable : variableList.getVariables()){
			queryWithReplacements = queryWithReplacements.replace("$" + variable.getName(), variable.getValue());
		}
		return queryWithReplacements;
	}
	
	/**
	 * Checks if a query contains a variable to replace.
	 * @param query The query string.
	 * @return true if a variable exist.
	 */
	public boolean containsVariable(String query) {
		return query.contains("$");
	}
}
