package de.neusta.imonitor.correlation.threadpool;

/*
 * #%L
 * correlation
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;

import javax.annotation.PostConstruct;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import de.neusta.imonitor.correlation.configs.CorrelationConfig;
import de.neusta.imonitor.correlation.processors.CorrelationEventProcessorTask;

@Component
@Scope("singleton")
public class CorrelationThreadPool {
	
	private static final int TIMEOUT_IN_SECONDS = 120;

	private static Logger PERFORMANCE_LOGGER = LoggerFactory.getLogger("performanceLogger");
	private static final Logger LOGGER = LoggerFactory.getLogger(CorrelationThreadPool.class);
	
	@Autowired
	private CorrelationConfig correlationConfig;

	private ExecutorService threadPool;

	@PostConstruct
	public void init() {
		int cores = correlationConfig.getCores();
		threadPool = Executors.newFixedThreadPool(cores);
		PERFORMANCE_LOGGER.info("Correlation uses " + cores + " threads/cores.");
	}

	public void execute(CorrelationEventProcessorTask correlationTask) {
		threadPool.execute(correlationTask);
	}
	
	public void waitForTasks() {
		threadPool.shutdown();
		try {
			threadPool.awaitTermination(TIMEOUT_IN_SECONDS, TimeUnit.SECONDS);
		} catch (Exception e) {
			LOGGER.error("Unexpected exception while waiting for threads to finish.", e);
		}
	}

}
