package de.neusta.imonitor.correlation.transformer;

/*
 * #%L
 * correlation
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.json.simple.JSONObject;
import org.springframework.stereotype.Service;

import de.neusta.imonitor.correlation.model.VariableNode;

@Service
public class JsonToVariableGraphTransformer {

	/**
	 * Transforms a json object to variable graph.
	 * 
	 * @param object
	 *            the json object to parse.
	 * @return the root node of the variable graph.
	 */
	public VariableNode transform(Object object) {
		VariableNode result = new VariableNode();
		internalTransform(object, result);
		return result;
	}

	@SuppressWarnings("unchecked")
	private void internalTransform(Object object, VariableNode currentNode) {
		if (object instanceof JSONObject) {
			JSONObject jsonObject = (JSONObject) object;
			Set<Map.Entry<String, Object>> entrySet = jsonObject.entrySet();
			for (Entry<String, Object> entry : entrySet) {
				String stringKey = entry.getKey();
				if (entry.getValue() instanceof String) {
					String stringValue = (String) entry.getValue();
					currentNode.getValues().put(stringKey, stringValue);
				} else if (entry.getValue() instanceof JSONObject) {
					Object childObject = entry.getValue();
					VariableNode newChild = new VariableNode();
					currentNode.getChildren().put(stringKey, newChild);
					internalTransform(childObject, newChild);
				}
			}
		}
	}

}
