package de.neusta.imonitor.correlation.utils;

/*
 * #%L
 * correlation
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import java.util.HashMap;
import java.util.List;
import java.util.Map.Entry;

import org.apache.commons.lang3.StringUtils;

import de.neusta.imonitor.ontology.Variable;
import de.neusta.imonitor.ontology.VariableList;

public class ConversionUtils {

	/**
	 * Removes the namespace prefix of an ontology uri.
	 * @param uri (e.g., imonitor:someIndividual)
	 * @return
	 */
	public static String removeNamespace(String uri){
		String result = uri;
		if (uri != null && result.contains(":")){
			result = result.substring(result.indexOf(":")+ 1);
		}
		return result;
	}
	
	
	/**
	 * Formats a list of strings to a single string translation in the form [value, value2, value3].
	 * @param list
	 * @return
	 */
	public static String formatList(List<String> list) {
		String result = "";
		if (list != null && list.size() > 0) {
			if (list.size() == 1) {
				result = "'" + list.get(0) + "'";
			} else
			{
				result = "[" + getCommaSeperatedList(list) + "]";
			}
		} else {
			result = "-";
		}
		return result;
	}

	/**
	 * Converts a simple list of strings to a single string translation by comma seperating the values.
	 * @param list
	 * @return
	 */
	public static String getCommaSeperatedList(List<String> list) {
		String result = "";
		for (int i = 0; i < list.size(); i++) {
			if (i == 0)
			{
				result += list.get(i);
			} else {
				result += ", " + list.get(i);
			}
		}
		return result;
	}
	
	/**
	 * Replaces variable in a string by the given variableList and a given actionQueryVaraibles list.
	 * The variableList variables are assumed to have a $-prefix in the string to replace.
	 * The actionQueryVariables are assumed to have a ?-prefix in the string to replace.
	 * @param variableList
	 * @param actionQueryVariables
	 * @param text
	 * @return
	 */
	public static String replaceVariablesInString(VariableList variableList, HashMap<String, List<String>> actionQueryVariables, String text) {
		String textResult = text;
		if (textResult == null) {
			return text;
		}
		// replace variables from the variable list (input data)
		for (Variable variable : variableList.getVariables()) {
			String variableName = "$" + variable.getName();
			String variableValue = ConversionUtils.removeNamespace(variable.getValue());
			if (variableValue == null) {
				variableValue = StringUtils.EMPTY;
			}
			textResult = textResult.replace(variableName, variableValue);
		}
		// replace variables from the action query variable list
		for (Entry<String, List<String>> variable : actionQueryVariables
				.entrySet()) {
			textResult = textResult.replace(
					"?" + variable.getKey(),
					ConversionUtils.formatList(variable.getValue()));
		}
		return textResult;
	}
}
