package de.neusta.imonitor.correlation;

/*
 * #%L
 * correlation
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.runners.MockitoJUnitRunner;

import de.neusta.imonitor.correlation.configs.CorrelationConfig;
import de.neusta.imonitor.correlation.configs.InputConfig;
import de.neusta.imonitor.correlation.garbagecollection.IndividualGarbageCollector;
import de.neusta.imonitor.correlation.input.IInputProcessor;
import de.neusta.imonitor.correlation.input.InputProcessorFactory;
import de.neusta.imonitor.correlation.ruleloader.RuleLoader;
import de.neusta.imonitor.correlation.services.OntologyInitService;
import de.neusta.imonitor.correlation.threadpool.IndividualGarbageCollectionThreadPool;
import de.neusta.imonitor.ontology.configs.OntologyConfig;


@RunWith(MockitoJUnitRunner.class)
public class CorrelationStartupTest {

	@InjectMocks
	private CorrelationStartup correlationStartup;
	
	@Mock
	private InputConfig inputConfig;
	@Mock
	private CorrelationConfig correlationConfig;
	@Mock
	private OntologyConfig ontologyConfig;
	@Mock
	private InputProcessorFactory inputProcessorFactory;
	@Mock
	private IInputProcessor inputProcessor;
	@Mock
	private Thread thread;
	@Mock
	private OntologyInitService ontologyInitService;
	@Mock
	private RuleLoader ruleLoader;
	@Mock
	private IndividualGarbageCollectionThreadPool individualGarbageCollectionThreadPool;
	@Mock
	private IndividualGarbageCollector individualGarbageCollector;

	
	@Test
	public void testProcessEvents() throws Exception {
		Mockito.when(inputConfig.check()).thenReturn(true);
		Mockito.when(correlationConfig.check()).thenReturn(true);
		Mockito.when(ontologyConfig.check()).thenReturn(true);
		Mockito.when(inputProcessorFactory.createInputProcessor()).thenReturn(inputProcessor);
		Mockito.when(inputProcessor.process()).thenReturn(thread);
		
		correlationStartup.start();
		
		Mockito.verify(inputProcessor).process();
		Mockito.verify(individualGarbageCollectionThreadPool).execute(individualGarbageCollector);
	}

}
