package de.neusta.imonitor.correlation.factory;

/*
 * #%L
 * correlation
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.runners.MockitoJUnitRunner;

import com.hp.hpl.jena.ontology.Individual;

import static org.junit.Assert.assertThat;

import static org.hamcrest.CoreMatchers.is;
import static org.hamcrest.Matchers.notNullValue;
import de.neusta.imonitor.ontology.Ontology;
import de.neusta.imonitor.ontology.constants.NamespaceConstants;


@RunWith(MockitoJUnitRunner.class)
public class IndividualFactoryTest {
	
	private static final String IMONITOR_OPERATION_IRI = "imonitor:Operation";
	
	@InjectMocks
	private IndividualFactory individualFactory;
	@Mock
	private Ontology ontology;

	@Test
	public void testCreateChild() throws Exception {
		Individual individual = Mockito.mock(Individual.class);
		Mockito.when(ontology.createIndividual(Mockito.anyString(), Mockito.eq(IMONITOR_OPERATION_IRI))).thenReturn(individual);
		
		String result = individualFactory.createChild("parentIri", "key");
		
		assertThat(result, is(notNullValue()));
		Mockito.verify(ontology).addDataProperty(Mockito.anyString(), Mockito.eq("imonitor:hasName"), Mockito.eq("key"));
		Mockito.verify(ontology).createRelation(Mockito.anyString(), Mockito.eq("parentIri"), Mockito.eq("imonitor:childOf"));
	}

	@Test
	public void testCreateDataRootIndividual() throws Exception {
		Individual individual = Mockito.mock(Individual.class);
		Mockito.when(ontology.createIndividual("imonitor:dataguid", IMONITOR_OPERATION_IRI)).thenReturn(individual);
		
		String result = individualFactory.createDataRootIndividual("dataguid");
		
		assertThat(result, is(NamespaceConstants.DEFAULT_NAMESPACE + "dataguid"));
	}

	@Test
	public void testCreateValueIndividual() throws Exception {
		Individual individual = Mockito.mock(Individual.class);
		Mockito.when(ontology.createIndividual(Mockito.anyString(), Mockito.eq(IMONITOR_OPERATION_IRI))).thenReturn(individual );

		
		String result = individualFactory.createValueIndividual("parentIri", "key", "value");
		
		assertThat(result, is(notNullValue()));
		Mockito.verify(ontology).createRelation(Mockito.anyString(), Mockito.eq("parentIri"), Mockito.eq("imonitor:childOf"));
		Mockito.verify(ontology).addDataProperty(Mockito.anyString(), Mockito.eq("imonitor:hasName"), Mockito.eq("key"));
		Mockito.verify(ontology).addDataProperty(Mockito.anyString(), Mockito.eq("imonitor:hasValue"), Mockito.eq("value"));
	}

}
