package de.neusta.imonitor.correlation.garbagecollection;

/*
 * #%L
 * correlation
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import java.util.List;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.internal.util.reflection.Whitebox;
import org.mockito.runners.MockitoJUnitRunner;

import static org.junit.Assert.assertThat;

import static org.hamcrest.CoreMatchers.is;


@RunWith(MockitoJUnitRunner.class)
public class IndividualGarbageCollectorTest {
	
	@InjectMocks
	private IndividualGarbageCollector classUnderTest;

	@Test
	public void testDeleteIndividual() throws Exception {
		classUnderTest.deleteIndividual("iri");
		
		List<String> garbage = (List<String>) Whitebox.getInternalState(classUnderTest, "garbage");
		assertThat(garbage.size(), is(1));
		assertThat(garbage.get(0), is("iri"));
	}

	@Test
	public void testCorrelationRunning() throws Exception {
		classUnderTest.correlationRunning();
		
		int runningCorrelations = (int) Whitebox.getInternalState(classUnderTest, "runningCorrelations");
		assertThat(runningCorrelations, is(1));
	}
	
	@Test
	public void testCorrelationRunningTwice() throws Exception {
		classUnderTest.correlationRunning();
		classUnderTest.correlationRunning();
		
		int runningCorrelations = (int) Whitebox.getInternalState(classUnderTest, "runningCorrelations");
		assertThat(runningCorrelations, is(2));
	}

	@Test
	public void testCorrelationFinished() throws Exception {
		classUnderTest.correlationRunning();
		
		classUnderTest.correlationFinished();
		
		int runningCorrelations = (int) Whitebox.getInternalState(classUnderTest, "runningCorrelations");
		assertThat(runningCorrelations, is(0));
	}

	@Test
	public void testCorrelationFinishedTwoFinishedOne() throws Exception {
		classUnderTest.correlationRunning();
		classUnderTest.correlationRunning();
		
		classUnderTest.correlationFinished();
		
		int runningCorrelations = (int) Whitebox.getInternalState(classUnderTest, "runningCorrelations");
		assertThat(runningCorrelations, is(1));
	}


}
