package de.neusta.imonitor.correlation.input;

/*
 * #%L
 * correlation
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import java.util.ArrayList;
import java.util.List;

import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.invocation.InvocationOnMock;
import org.mockito.runners.MockitoJUnitRunner;
import org.mockito.stubbing.Answer;

import de.neusta.imonitor.correlation.model.VariableNode;
import de.neusta.imonitor.correlation.processors.CorrelationEventProcessor;
import de.neusta.imonitor.correlation.service.OntologyVariableService;
import de.neusta.imonitor.correlation.services.DefaultVariablesService;
import de.neusta.imonitor.correlation.services.ValueMappingService;
import de.neusta.imonitor.correlation.transformer.JsonToVariableGraphTransformer;
import de.neusta.imonitor.ontology.VariableList;


@RunWith(MockitoJUnitRunner.class)
public class JSONProcessorTest {
	
	@InjectMocks
	private JSONProcessor jsonProcessor;
	@Mock
	private DefaultVariablesService defaultVariablesService;
	@Mock
	private JsonToVariableGraphTransformer jsonToVariableGraphTransformer;
	@Mock
	private ValueMappingService valueMappingService;
	@Mock
	private JSONParser jsonParser;
	@Mock
	private OntologyVariableService ontologyVariableService;
	@Mock
	private CorrelationEventProcessor correlationEventProcessor;


	@Test
	public void testProcessLine() throws Exception {
		final String LINE_TO_PARSE = "{ \"var\":\"val\" }";
		JSONObject jsonObject = Mockito.mock(JSONObject.class);
		Mockito.when(jsonParser.parse(LINE_TO_PARSE)).thenReturn(jsonObject );
		VariableNode rootVariableNode = Mockito.mock(VariableNode.class);
		Mockito.when(jsonToVariableGraphTransformer.transform(Mockito.eq(jsonObject))).thenReturn(rootVariableNode );
		Mockito.doAnswer(new Answer<Void>() {
			@Override
			public Void answer(InvocationOnMock invocation) throws Throwable {
				VariableList variableList = (VariableList) invocation.getArguments()[0];
				variableList.createVariable("DATA", "guid");
				return null;
			}
		}).when(defaultVariablesService).addDefaultVariables(Mockito.any(VariableList.class));
		List<String> individuals = new ArrayList<String>();
		individuals.add("individualUri");
		Mockito.when(ontologyVariableService.addToOntology(rootVariableNode, "guid")).thenReturn(individuals );
		
		jsonProcessor.processLine(LINE_TO_PARSE);
	}

}
