package de.neusta.imonitor.correlation.processors;

/*
 * #%L
 * correlation
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import static org.junit.Assert.*;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.runners.MockitoJUnitRunner;

import de.neusta.imonitor.correlation.services.CommandService;
import de.neusta.imonitor.correlation.services.IncidentService;
import de.neusta.imonitor.ontology.Variable;
import de.neusta.imonitor.ontology.VariableList;
import de.neusta.imonitor.ruleparser.abstraction.Abstraction;
import de.neusta.imonitor.ruleparser.rules.Action;
import de.neusta.imonitor.ruleparser.rules.Rule;


@RunWith(MockitoJUnitRunner.class)
public class ActionProcessorTest {
	
	@InjectMocks
	private ActionProcessor actionProcessor;
	@Mock
	private CommandService commandService;
	@Mock
	private ExplanationProcessor explanationProcessor;
	@Mock
	private QueryProcessor queryProcessor;
	@Mock
	private IncidentService incidentService;
	@Mock
	private RecommendationProcessor recommendationProcessor;
	@Mock
	private UpdateProcessor updateProcessor;
	@Captor
	private ArgumentCaptor<VariableList> variableListCaptor;


	@Test
	public void testProcessActionCallsProcessCommand() throws Exception {
		Abstraction abstraction = null;
		VariableList variableList = new VariableList();
		Action action = new Action();
		action.setCommand("command");
		Rule rule = new Rule();
		rule.setAction(action);
		
		actionProcessor.processAction(rule, variableList, abstraction);
		
		Mockito.verify(commandService).processCommand(Mockito.eq("command"), Mockito.anyListOf(String.class), Mockito.anyListOf(String.class), Mockito.eq(variableList));
	}


	@Test
	public void testProcessAction() throws Exception {
		Abstraction abstraction = null;
		VariableList variableList = new VariableList();
		Action action = new Action();
		action.setCommand("command");
		action.setIncidentName("incidentname");
		action.setRisk(5);
		Rule rule = new Rule();
		rule.setAction(action);
		
		actionProcessor.processAction(rule, variableList, abstraction);
		
		Mockito.verify(commandService).processCommand(Mockito.eq("command"), Mockito.anyListOf(String.class), Mockito.anyListOf(String.class), variableListCaptor.capture());
		VariableList capturedVariableList = variableListCaptor.getValue();
		Variable incidentNameVariable = capturedVariableList.getVariable("incidentname");
		assertEquals("incidentname", incidentNameVariable.getValue());
		Variable riskVariable = capturedVariableList.getVariable("risk");
		assertEquals("5", riskVariable.getValue());
		
	}
	
}
