package de.neusta.imonitor.correlation.processors;

/*
 * #%L
 * correlation
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import java.util.Arrays;
import java.util.List;
import java.util.UUID;

import org.apache.log4j.PropertyConfigurator;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.kubek2k.springockito.annotations.ReplaceWithMock;
import org.kubek2k.springockito.annotations.SpringockitoContextLoader;
import org.mockito.Mockito;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.AbstractJUnit4SpringContextTests;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import de.neusta.imonitor.correlation.garbagecollection.IndividualGarbageCollector;
import de.neusta.imonitor.correlation.model.VariableNode;
import de.neusta.imonitor.correlation.threadpool.CorrelationThreadPool;
import de.neusta.imonitor.correlation.threadpool.IndividualGarbageCollectionThreadPool;
import de.neusta.imonitor.ontology.VariableList;
import de.neusta.imonitor.ontology.exception.VariableAlreadyExistsException;
import de.neusta.imonitor.ruleparser.rules.Condition;
import de.neusta.imonitor.ruleparser.rules.Rule;
import de.neusta.imonitor.ruleparser.rules.RuleParser;

import static org.junit.Assert.fail;

/**
 * Concurrency tests.
 */

@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration(loader = SpringockitoContextLoader.class, locations = "classpath:/test-application.xml")
@DirtiesContext(classMode = DirtiesContext.ClassMode.AFTER_CLASS)
public class ConcurrentCorrelationEventProcessorTaskTest extends AbstractJUnit4SpringContextTests {

	private static final float NUMBER_OF_TESTS = 500f;
	private static final int NUMBER_OF_VARIABLES = 10;
	private static final int NUMBER_OF_VARIABLENODES = 10;
	
	private Logger PERFORMANCE_LOGGER = LoggerFactory.getLogger("performanceLogger");

	@Autowired
	CorrelationThreadPool correlationThreadPool;
	@Autowired
	private ApplicationContext applicationContext;

	@Autowired
	@ReplaceWithMock
	private RuleParser ruleParser;
	
	@Autowired
	private IndividualGarbageCollectionThreadPool individualGarbageCollectionThreadPool;
	@Autowired
	private IndividualGarbageCollector individualGarbageCollector;
	
	@Before
	public void setup() {
		PropertyConfigurator.configure("../configs/log4j.properties");
		individualGarbageCollectionThreadPool.execute(individualGarbageCollector);
	}

	@Test
	public void testConcurrentTasks() throws Exception {
		Rule rule = new Rule();
		Condition condition = new Condition();
		condition.setQuery("ASK { ?x a ?y. }");
		rule.setCondition(condition);
		List<Rule> rules = Arrays.asList(rule);
		Mockito.when(ruleParser.getRules()).thenReturn(rules);
		long start = System.currentTimeMillis();
		System.out.println("Starting threads...");
		for (int i = 0; i < NUMBER_OF_TESTS; i++) {
			CorrelationEventProcessorTask correlationTask = applicationContext.getBean(CorrelationEventProcessorTask.class);
			createVariableNodes(correlationTask);
			createVariables(correlationTask);
			correlationThreadPool.execute(correlationTask);
		}
		correlationThreadPool.waitForTasks();
		
		long finish = System.currentTimeMillis();
		Thread.sleep(1000);
		long duration = finish - start;
		float durationPerTest = duration / NUMBER_OF_TESTS ;
		PERFORMANCE_LOGGER.info(NUMBER_OF_TESTS + " tests took " + duration + " ms, i.e. " +durationPerTest + " ms per test.");
		if (durationPerTest > 100) {
			fail("correlation is too time expensive!");
		}
	}

	private void createVariableNodes(CorrelationEventProcessorTask correlationTask) {
		VariableNode variableGraphRoot = new VariableNode();
		for (int i = 0; i < NUMBER_OF_VARIABLENODES; i++) {
			VariableNode varNode = new VariableNode();
			variableGraphRoot.getChildren().put("testvar" + UUID.randomUUID().toString(), varNode);
			correlationTask.setVariableGraph(variableGraphRoot);
		}
	}

	private void createVariables(CorrelationEventProcessorTask correlationTask) throws VariableAlreadyExistsException {
		String uuid = UUID.randomUUID().toString();
		VariableList variableList = new VariableList();
		variableList.createVariable("DATA", uuid);
		correlationTask.setVariableList(variableList);
		for (int i = 0; i < NUMBER_OF_VARIABLES; i++) {
			variableList.createVariable("var" + UUID.randomUUID().toString(), "value");
		}
	}

}
