package de.neusta.imonitor.correlation.processors;

/*
 * #%L
 * correlation
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import java.util.ArrayList;
import java.util.List;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.runners.MockitoJUnitRunner;

import de.neusta.imonitor.correlation.garbagecollection.IndividualGarbageCollector;
import de.neusta.imonitor.correlation.model.VariableNode;
import de.neusta.imonitor.correlation.service.OntologyVariableService;
import de.neusta.imonitor.ontology.VariableList;
import de.neusta.imonitor.ruleparser.rules.RuleParser;

@RunWith(MockitoJUnitRunner.class)
public class CorrelationEventProcessorTaskTest {
	
	@InjectMocks
	private CorrelationEventProcessorTask correlationEventProcessorTask;
	@Mock
	private OntologyVariableService ontologyVariableService;
	@Mock
	private RuleParser ruleParser;
	@Mock
	private IndividualGarbageCollector individualGarbageCollector;
	
	@Mock
	private VariableNode rootVariableNode;

	@Test
	public void testRunAddsTemporaryVariablesToOntology() throws Exception {
		VariableList variableList = new VariableList();
		variableList.createVariable("DATA", "guid");
		List<String> createdIndividuals = new ArrayList<String>();
		Mockito.when(ontologyVariableService.addToOntology(rootVariableNode, "guid")).thenReturn(createdIndividuals);
		correlationEventProcessorTask.setVariableGraph(rootVariableNode);
		correlationEventProcessorTask.setVariableList(variableList );
		
		correlationEventProcessorTask.run();
		
		Mockito.verify(ontologyVariableService).addToOntology(rootVariableNode, "guid");
		Mockito.verify(ontologyVariableService).removeFromOntology(createdIndividuals);
		Mockito.verify(individualGarbageCollector).correlationRunning();
		Mockito.verify(individualGarbageCollector).correlationFinished();
	}

}
