package de.neusta.imonitor.correlation.service;

/*
 * #%L
 * correlation
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import java.util.ArrayList;
import java.util.List;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.runners.MockitoJUnitRunner;

import com.hp.hpl.jena.ontology.Individual;

import de.neusta.imonitor.correlation.factory.IndividualFactory;
import de.neusta.imonitor.correlation.garbagecollection.IndividualGarbageCollector;
import de.neusta.imonitor.correlation.model.VariableNode;
import de.neusta.imonitor.ontology.Ontology;
import static org.hamcrest.CoreMatchers.is;

import static org.junit.Assert.assertThat;


@RunWith(MockitoJUnitRunner.class)
public class OntologyVariableServiceTest {
	
	private static final String DATA_GUID = "guid";
	
	@InjectMocks
	private OntologyVariableService ontologyVariableService;
	@Mock
	private Ontology ontology;
	@Mock
	private IndividualFactory individualFactory;
	@Mock
	private IndividualGarbageCollector individualGarbageCollector;
	@Mock
	private Individual rootIndividual;
	
	@Before
	public void setUp() {
		Mockito.when(individualFactory.createDataRootIndividual(DATA_GUID)).thenReturn("rootUri");
	}

	@Test
	public void testAddToOntologySingleValue() throws Exception {
		VariableNode variableNode = new VariableNode();
		variableNode.getValues().put("var", "val");
		Mockito.when(individualFactory.createValueIndividual("imonitor:" + DATA_GUID, "var", "val")).thenReturn("individual1");
		
		List<String> result = ontologyVariableService.addToOntology(variableNode, DATA_GUID);
		
		assertThat(result.size(), is(2));
		assertThat(result.get(0), is("rootUri"));
		assertThat(result.get(1), is("individual1"));
	}

	@Test
	public void testAddToOntologyTwoValues() throws Exception {
		VariableNode variableNode = new VariableNode();
		variableNode.getValues().put("var", "val");
		variableNode.getValues().put("var2", "val2");
		Mockito.when(individualFactory.createValueIndividual("imonitor:" + DATA_GUID, "var", "val")).thenReturn("individual1");
		Mockito.when(individualFactory.createValueIndividual("imonitor:" + DATA_GUID, "var2", "val2")).thenReturn("individual2");
		
		List<String> result = ontologyVariableService.addToOntology(variableNode, DATA_GUID);
		
		assertThat(result.size(), is(3));
		assertThat(result.get(0), is("rootUri"));
		assertThat(result.get(1), is("individual1"));
		assertThat(result.get(2), is("individual2"));
	}

	@Test
	public void testAddToOntologyNestedValues() throws Exception {
		VariableNode variableNode = new VariableNode();
		VariableNode childNode = new VariableNode();
		childNode.getValues().put("var", "val");
		variableNode.getChildren().put("child", childNode);
		Individual childIndividual = Mockito.mock(Individual.class);
		Mockito.when(childIndividual.getURI()).thenReturn("childUri");
		Mockito.when(individualFactory.createChild("imonitor:" + DATA_GUID,  "child")).thenReturn("childUri");
		Mockito.when(individualFactory.createValueIndividual("childUri", "var", "val")).thenReturn("valueUri");
		
		List<String> result = ontologyVariableService.addToOntology(variableNode, DATA_GUID);
		
		assertThat(result.size(), is(3));
		assertThat(result.get(0), is("rootUri"));
		assertThat(result.get(1), is("childUri"));
		assertThat(result.get(2), is("valueUri"));
	}

	@Test
	public void testRemoveFromOntology() throws Exception {
		List<String> individuals = new ArrayList<String>();
		individuals.add("individual1");
		individuals.add("individual2");
		
		ontologyVariableService.removeFromOntology(individuals);
		
		Mockito.verify(individualGarbageCollector).deleteIndividual("individual1");
		Mockito.verify(individualGarbageCollector).deleteIndividual("individual2");
	}

}
