package de.neusta.imonitor.correlation.services;

/*
 * #%L
 * correlation
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import static org.junit.Assert.assertEquals;

import java.util.ArrayList;
import java.util.List;

import javax.transaction.Transactional;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import de.neusta.imonitor.correlation.persistency.entities.IncidentEntity;
import de.neusta.imonitor.correlation.persistency.repositories.IncidentRepository;
import de.neusta.imonitor.ontology.VariableList;
import de.neusta.imonitor.ruleparser.rules.Action;

@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration("/test-application.xml")
public class IncidentServiceTest {

	@Autowired
	private IncidentService actionDatabaseHelper;
	
	@Autowired
	private IncidentRepository incidentRepository;


	@Test
	public void testWriteIncidentToDatabase() throws Exception {
		writeToDB();
		readFromDBAndAssert();
	}


	@Transactional
	public void readFromDBAndAssert() {
		List<IncidentEntity> incidentEntities = incidentRepository.findAll();
		assertEquals(1, incidentEntities.size());
		IncidentEntity incidentEntity = incidentEntities.get(0);
		assertEquals(5, incidentEntity.getRisk());
		assertEquals(2, incidentEntity.getExplanations().size());
		assertEquals(2, incidentEntity.getRecommendations().size());
	}


	@Transactional
	public void writeToDB() {
		VariableList variableList = new VariableList();
		List<String> explanations = new ArrayList<String>();
		explanations.add("explanation_1");
		explanations.add("explanation_2");
		List<String> recommendations = new ArrayList<String>();
		recommendations.add("recommendation_1");
		recommendations.add("recommendation_2");
		Action action = new Action();
		action.setRisk(5);
		actionDatabaseHelper.writeIncidentToDatabase(action, variableList,
				explanations, recommendations);
	}
	
}
