package de.neusta.imonitor.correlation.services;

/*
 * #%L
 * correlation
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import static org.junit.Assert.*;

import java.util.ArrayList;
import java.util.List;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.invocation.InvocationOnMock;
import org.mockito.runners.MockitoJUnitRunner;
import org.mockito.stubbing.Answer;

import de.neusta.imonitor.ontology.VariableList;

@RunWith(MockitoJUnitRunner.class)
public class MakroServiceTest {

	@InjectMocks
	private MakroService makroService;
	@Mock
	private VariableReplacementService variableReplacementService;
	@Mock
	private StringListReplacementService stringListReplacementService;

	private List<String> explanations;
	private List<String> recommendations;
	private VariableList variableList;

	@Before
	public void setup() {
		explanations = new ArrayList<String>();
		recommendations = new ArrayList<String>();
		variableList = new VariableList();
		Mockito.when(
				stringListReplacementService.replaceAll(Mockito.anyString(),
						Mockito.anyString(), Mockito.anyListOf(String.class)))
				.thenAnswer(new Answer<String>() {

					@Override
					public String answer(InvocationOnMock invocation)
							throws Throwable {
						Object[] args = invocation.getArguments();
						return (String) args[0];
					}
				});
		
		Mockito.when(
				variableReplacementService
						.replace("cmd $var cmd", variableList)).thenAnswer(new Answer<String>() {

							@Override
							public String answer(InvocationOnMock invocation)
									throws Throwable {
								Object[] args = invocation.getArguments();
								return (String) args[0];
							}
						});
	}

	@Test
	public void testReplacesVariable() throws Exception {
		variableList.createVariable("var", "varvalue");
		Mockito.when(
				variableReplacementService
						.replace("cmd $var cmd", variableList)).thenReturn(
				"cmd varvalue cmd");

		String result = makroService.substitute("cmd $var cmd", explanations,
				recommendations, variableList);

		assertEquals("cmd varvalue cmd", result);
	}

	@Test
	public void testReplacesExplanations() throws Exception {
		Mockito.when(
				stringListReplacementService.replaceAll("cmd $var cmd",
						"$explanation", explanations)).thenReturn(
				"replaced_explanations");

		String result = makroService.substitute("cmd $var cmd", explanations,
				recommendations, variableList);

		assertEquals("replaced_explanations", result);
	}

	@Test
	public void testReplacesRecommendations() throws Exception {
		Mockito.when(
				stringListReplacementService.replaceAll("cmd $var cmd",
						"$recommendation", recommendations)).thenReturn(
				"replaced_recommendations");

		String result = makroService.substitute("cmd $var cmd", recommendations,
				recommendations, variableList);

		assertEquals("replaced_recommendations", result);
	}

	
}
