package de.neusta.imonitor.correlation.services;

/*
 * #%L
 * correlation
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.runners.MockitoJUnitRunner;

import de.neusta.imonitor.ontology.VariableList;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

@RunWith(MockitoJUnitRunner.class)
public class VariableReplacementServiceTest {

	@InjectMocks
	private VariableReplacementService variableReplacementService;

	@Test
	public void testReplace() throws Exception {
		String query = "$test";
		VariableList variableList = new VariableList();
		variableList.createVariable("test", "testvalue");

		String queryResult = variableReplacementService.replace(query,
				variableList);

		assertEquals("testvalue", queryResult);
	}
	
	@Test
	public void testReplaceTwoVariables() throws Exception {
		String query = "ASK { $var owl:sameAs $othervar }";
		VariableList variableList = new VariableList();
		variableList.createVariable("var", "testvalue");
		variableList.createVariable("othervar", "testvalue2");

		String queryResult = variableReplacementService.replace(query,
				variableList);

		assertEquals("ASK { testvalue owl:sameAs testvalue2 }", queryResult);
	}

	@Test
	public void testContainsVariable() throws Exception {
		boolean result = variableReplacementService.containsVariable("{ $somevar }");
		
		assertTrue(result);
	}
	
	@Test
	public void testContainsVariableFalse() throws Exception {
		boolean result = variableReplacementService.containsVariable("{ ?some_internal_var }");
		
		assertFalse(result);
	}
	

}
