package de.neusta.imonitor.correlation.transformer;

/*
 * #%L
 * correlation
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import static org.hamcrest.CoreMatchers.is;
import static org.junit.Assert.assertThat;

import org.json.simple.parser.JSONParser;
import org.json.simple.parser.ParseException;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.runners.MockitoJUnitRunner;

import de.neusta.imonitor.correlation.model.VariableNode;


@RunWith(MockitoJUnitRunner.class)
public class JsonToVariableGraphTransformerTest {
	
	@InjectMocks
	private JsonToVariableGraphTransformer jsonToVariableGraphTransformer;

	@Test
	public void testTransformOneElement() throws Exception {
		Object jsonObject = createJsonObject("{\"var\":\"val\"}");
		
		VariableNode result = jsonToVariableGraphTransformer.transform(jsonObject);
		
		assertThat(result.getValues().size(), is(1));
		assertThat(result.getValues().get("var"), is("val"));
	}

	@Test
	public void testTransformTwoElements() throws Exception {
		Object jsonObject = createJsonObject("{\"var\":\"val\", \"var2\":\"val2\"}");
		
		VariableNode result = jsonToVariableGraphTransformer.transform(jsonObject);
		
		assertThat(result.getValues().size(), is(2));
		assertThat(result.getValues().get("var"), is("val"));
		assertThat(result.getValues().get("var2"), is("val2"));
	}
	
	@Test
	public void testTransformWithChild() throws Exception {
		Object jsonObject = createJsonObject("{ \"child\": {\"var\":\"val\"} }");
		
		VariableNode result = jsonToVariableGraphTransformer.transform(jsonObject);
		
		assertThat(result.getChildren().size(), is(1));
		VariableNode child = result.getChildren().get("child");
		assertThat(child.getValues().get("var"), is("val"));
	}
	
	@Test
	public void testTransformWithTwoLevelsofChildren() throws Exception {
		Object jsonObject = createJsonObject("{ \"child\": {\"var\": { \"var2\":\"val2\"  } } }");
		
		VariableNode result = jsonToVariableGraphTransformer.transform(jsonObject);
		
		assertThat(result.getChildren().size(), is(1));
		VariableNode child = result.getChildren().get("child");
		VariableNode childSecondLevel = child.getChildren().get("var");
		assertThat(childSecondLevel.getValues().get("var2"), is("val2"));
	}

	
	
	private Object createJsonObject(String jsonData) throws ParseException {
		JSONParser jsonParser = new JSONParser();
		Object jsonObject = jsonParser.parse(jsonData);
		return jsonObject;
	}


}
