package de.neusta.imonitor.correlation;

import junit.framework.Assert;

import org.junit.Test;

import de.neusta.imonitor.correlation.eventprocessor.EventProcessorFactory;
import de.neusta.imonitor.correlation.eventprocessor.IEventProcessor;
import de.neusta.imonitor.correlation.mocks.ActionProcessorMock;
import de.neusta.imonitor.ontology.Ontology;
import de.neusta.imonitor.ontology.VariableList;

public class CorrelationEventProcessorTests {
	
	
	@Test
	public void testCorrelationEventProcessorStartsWithoutVariableList() {
		Ontology ontology = TestHelper.getTestOntology();
		String ruleFile = TestHelper.getTestRuleFile();
		ActionProcessorMock actionProcessorMock = new ActionProcessorMock();
		IEventProcessor eventProcessor = EventProcessorFactory.createEventProcessor(ontology, ruleFile, actionProcessorMock);
		VariableList variableList = new VariableList();
		eventProcessor.process(variableList);
		Assert.assertNotNull(actionProcessorMock.getAction());
	}
	
	@Test
	public void testCorrelationEventProcessorCallsActionProcessor() throws Exception{
		Ontology ontology = TestHelper.getTestOntology();
		String ruleFile = TestHelper.getTestRuleFile();
		ActionProcessorMock actionProcessorMock = new ActionProcessorMock();
		IEventProcessor eventProcessor = EventProcessorFactory.createEventProcessor(ontology, ruleFile, actionProcessorMock);
		VariableList variableList = new VariableList();
		variableList.createVariable("var1", "imonitor:myWebServer");
		eventProcessor.process(variableList);
		Assert.assertNotNull(actionProcessorMock.getAction());
		Assert.assertNotNull(actionProcessorMock.getAction().getCommand().equals("testcommand"));
	}

	@Test
	public void testCorrelationEventProcessorCannotMatch() throws Exception{
		Ontology ontology = TestHelper.getTestOntology();
		String ruleFile = TestHelper.getTestRuleFile();
		ActionProcessorMock actionProcessorMock = new ActionProcessorMock();
		IEventProcessor eventProcessor = EventProcessorFactory.createEventProcessor(ontology, ruleFile, actionProcessorMock);
		VariableList variableList = new VariableList();
		variableList.createVariable("var1", "imonitor:doesNotExist");
		eventProcessor.process(variableList);
		Assert.assertNull(actionProcessorMock.getAction());
	}
	
}
