package de.neusta.imonitor.correlation;

import junit.framework.Assert;

import org.junit.Test;

import de.neusta.imonitor.correlation.patternmatcher.Pattern;
import de.neusta.imonitor.correlation.patternmatcher.PatternMatcher;
import de.neusta.imonitor.ontology.VariableList;

public class PatternMatcherTests {

	@Test
	public void testPatternMatcherCreateInstance(){
		PatternMatcher patternMatcher = TestHelper.createPatternMatcherInstance();
		Assert.assertNotNull(patternMatcher);
	}
	
	@Test
	public void testPatternMatcherMatchesSimplePattern(){
		PatternMatcher patternMatcher = TestHelper.createPatternMatcherInstance();
		Pattern pattern = new Pattern("ASK  { imonitor:WebServer rdfs:subClassOf imonitor:Server; }");
		VariableList variableList = new VariableList();
		boolean result = patternMatcher.matchPattern(pattern, variableList);
		Assert.assertTrue(result);
	}
	
	@Test
	public void testPatternMatcherMatchesPatternWithVariable(){
		PatternMatcher patternMatcher = TestHelper.createPatternMatcherInstance();
		Pattern pattern = new Pattern("ASK  { $var rdfs:subClassOf imonitor:Server; }");
		VariableList variableList = new VariableList();
		try {
			variableList.createVariable("var", "imonitor:WebServer");
		} catch (Exception e) {
			Assert.fail();
		}
		boolean result = patternMatcher.matchPattern(pattern, variableList);
		Assert.assertTrue(result);
	}
	
	@Test
	public void testPatternMatcherDoesNotMatchPatternWithVariable(){
		PatternMatcher patternMatcher = TestHelper.createPatternMatcherInstance();
		Pattern pattern = new Pattern("ASK  { $var rdfs:subClassOf imonitor:Server; }");
		VariableList variableList = new VariableList();
		try {
			variableList.createVariable("var", "imonitor:NoWebServer");
		} catch (Exception e) {
			Assert.fail();
		}
		boolean result = patternMatcher.matchPattern(pattern, variableList);
		Assert.assertFalse(result);
	}

	
	@Test
	public void testPatternMatcherDoesMatchPatternWithWrongVariableName(){
		PatternMatcher patternMatcher = TestHelper.createPatternMatcherInstance();
		Pattern pattern = new Pattern("ASK  { $var rdfs:subClassOf imonitor:Server; }");
		VariableList variableList = new VariableList();
		try {
			variableList.createVariable("varWithWrongName", "imonitor:WebServer");
		} catch (Exception e) {
			Assert.fail();
		}
		boolean result = patternMatcher.matchPattern(pattern, variableList);
		Assert.assertTrue(result);
	}
}
