package de.neusta.imonitor.knowledgeserver.controller;

/*
 * #%L
 * knowledgeServer
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import java.io.Serializable;

import javax.annotation.PostConstruct;
import javax.faces.model.DataModel;
import javax.faces.model.ListDataModel;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Service;

import de.neusta.imonitor.knowledgeserver.persistence.service.RuleRepositoryService;
import de.neusta.imonitor.knowledgeserver.webservice.dto.PageDTO;

@Service
@Scope("session")
public class RuleController implements Serializable {
	
	private static final long serialVersionUID = 6480959124024303982L;

	private DataModel<Rule> rulesModel;
	
	@Autowired
	private RuleRepositoryService ruleService;
	
	private int page;

	private PageDTO<Rule> rulesPage;
	
	@PostConstruct
	public void init(){
		this.getRulesPage();
		this.rulesModel = new ListDataModel<Rule>(this.rulesPage.getContent());
	}
	
	public DataModel<Rule> getRulesPage(){
		this.rulesPage = this.ruleService.getRulesPage(this.page);
		return rulesModel = new ListDataModel<Rule>(this.rulesPage.getContent());
	}
	
	public long getRuleId(){
		Rule rule = rulesModel.getRowData();
		if (rule != null){
			return rule.getId();
		} else
		{
			return -1;
		}
	}
	
	public String delete(){
		Rule rule = rulesModel.getRowData();
		if (rule != null){
			this.ruleService.delete(rule.getId());
			this.rulesPage = ruleService.getRulesPage(this.page);
		}
		return "";
	}
	
	public String accept(){
		Rule rule = rulesModel.getRowData();
		if (rule != null){
			this.ruleService.accept(rule.getId());
		}
		return "";
	}
	
	public String decline(){
		Rule rule = rulesModel.getRowData();
		if (rule != null){
			this.ruleService.decline(rule.getId());
		}
		return "";
	}

	public boolean isNextPageAvailable() {
		return this.rulesPage.getNumber() + 1 < this.rulesPage.getTotalPages();
	}

	public boolean isPreviousPageAvailable() {
		return this.rulesPage.getNumber() > 0;
	}

	public int getPage() {
		return page;
	}

	public void nextPage() {
		this.page++;
	}

	public void previousPage() {
		this.page--;
	}
	
}
