package de.neusta.imonitor.knowledgeserver.persistence.service;

/*
 * #%L
 * knowledgeServer
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import java.util.ArrayList;
import java.util.List;

import javax.transaction.Transactional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;

import de.neusta.imonitor.knowledgeserver.controller.Rule;
import de.neusta.imonitor.knowledgeserver.converter.RuleEntityToRuleConverter;
import de.neusta.imonitor.knowledgeserver.converter.RuleEntityToRuleDtoConverter;
import de.neusta.imonitor.knowledgeserver.persistence.entities.RuleEntity;
import de.neusta.imonitor.knowledgeserver.persistence.entities.enums.Status;
import de.neusta.imonitor.knowledgeserver.persistence.repositories.RuleRepository;
import de.neusta.imonitor.knowledgeserver.webservice.dto.PageDTO;
import de.neusta.imonitor.knowledgeserver.webservice.dto.RuleDTO;

@Service
@Transactional
public class RuleRepositoryService {

	private static final int PAGE_SIZE = 10;

	@Autowired
	private RuleRepository ruleRepository;

	@Autowired
	private RuleEntityToRuleConverter ruleEntityToRuleConverter;

	@Autowired
	private RuleEntityToRuleDtoConverter ruleEntityToRuleDtoConverter;

	public List<Rule> getRules() {
		List<RuleEntity> allRules = this.ruleRepository.findAll();
		return this.convertToRules(allRules);
	}

	public void delete(long id) {
		this.ruleRepository.delete(id);
	}

	public RuleEntity saveRule(RuleEntity ruleEntity) {
		return this.ruleRepository.save(ruleEntity);
	}

	public List<RuleDTO> getAcceptedRuleDTOs() {
		List<RuleEntity> allRules = this.ruleRepository
				.findByStatus(Status.ACCEPTED);
		List<RuleDTO> ruleDTOs = new ArrayList<RuleDTO>();
		for (RuleEntity ruleEntity : allRules) {
			ruleDTOs.add(this.ruleEntityToRuleDtoConverter.convert(ruleEntity));
		}
		return ruleDTOs;
	}

	public void accept(long ruleId) {
		RuleEntity ruleEntity = this.ruleRepository.findOne(ruleId);
		ruleEntity.setStatus(Status.ACCEPTED);
		this.ruleRepository.save(ruleEntity);
	}

	public void decline(long ruleId) {
		RuleEntity ruleEntity = this.ruleRepository.findOne(ruleId);
		ruleEntity.setStatus(Status.DECLINED);
		this.ruleRepository.save(ruleEntity);
	}

	public PageDTO<Rule> getRulesPage(int pageIdx) {
		PageRequest pageRequest = new PageRequest(pageIdx, PAGE_SIZE,
				Sort.Direction.DESC, "name");
		Page<RuleEntity> rulePage = this.ruleRepository.findAll(pageRequest);

		List<RuleEntity> ruleEntities = rulePage.getContent();
		List<Rule> rules = convertToRules(ruleEntities);
		return new PageDTO<Rule>(rules, rulePage.getNumber(),
				rulePage.getTotalPages());
	}

	private List<Rule> convertToRules(List<RuleEntity> ruleEntities) {
		List<Rule> rules = new ArrayList<Rule>();
		for (RuleEntity ruleEntity : ruleEntities) {
			rules.add(this.ruleEntityToRuleConverter.convert(ruleEntity));
		}
		return rules;
	}

	public RuleEntity getRuleEntity(long ruleId) {
		return this.ruleRepository.findOne(ruleId);
	}

	public RuleDTO getAcceptedRuleDTO(Long id) {
		RuleEntity ruleEntity = this.ruleRepository.findOne(id);
		if (ruleEntity.getStatus() == Status.ACCEPTED) {
			RuleDTO ruleDTO = this.ruleEntityToRuleDtoConverter
					.convert(ruleEntity);
			return ruleDTO;
		} else {
			return null;
		}
	}
}
