package de.neusta.imonitor.knowledgeserver.webservice;

/*
 * #%L
 * knowledgeServer
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import java.util.Date;
import java.util.List;

import javax.jws.WebMethod;
import javax.jws.WebService;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.neusta.imonitor.knowledgeserver.converter.RuleDTOToRuleEntityConverter;
import de.neusta.imonitor.knowledgeserver.converter.RuleEntityToRuleDtoConverter;
import de.neusta.imonitor.knowledgeserver.ontology.OntologyProvider;
import de.neusta.imonitor.knowledgeserver.persistence.entities.RuleEntity;
import de.neusta.imonitor.knowledgeserver.persistence.entities.enums.Status;
import de.neusta.imonitor.knowledgeserver.persistence.service.RuleRepositoryService;
import de.neusta.imonitor.knowledgeserver.webservice.dto.RuleDTO;

/**
 * Webservice at http://localhost:8081/RuleService.
 * 
 * @author celfers
 * 
 */
@Service
@WebService(serviceName = "RuleService")
public class RuleService {

	private final static Logger LOGGER = LoggerFactory
			.getLogger(RuleService.class);

	@Autowired
	private RuleRepositoryService ruleRepositoryService;

	@Autowired
	private RuleDTOToRuleEntityConverter ruleDTOToRuleEntityConverter;

	@Autowired
	private RuleEntityToRuleDtoConverter ruleEntityToRuleDtoConverter;
	
	@Autowired
	private OntologyProvider ontologyProvider;

	@Autowired
	private RuleValidationService ruleValidator;

	@WebMethod
	public boolean commitRule(RuleDTO ruleDTO) {
		LOGGER.info("Validating comitted rule.");
		boolean valid = this.ruleValidator.validate(ruleDTO.getXml());
		if (valid) {
			LOGGER.info("Comitting rule to database.");
			RuleEntity ruleEntity = this.ruleDTOToRuleEntityConverter
					.convert(ruleDTO);
			ruleEntity.setStatus(Status.NEW);
			Date commitDate = new Date();
			ruleEntity.setCommitDate(commitDate );
			RuleEntity result = this.ruleRepositoryService.saveRule(ruleEntity);
			if (result != null) {
				LOGGER.info("Rule saved.");
				return true;
			} else {
				LOGGER.warn("Could not save rule.");
				return false;
			}
		} else
		{
			LOGGER.warn("Invalid rule xml committed.");
			return false;
		}
	}

	@WebMethod
	public List<RuleDTO> getRules() {
		return this.ruleRepositoryService.getAcceptedRuleDTOs();
	}
	
	@WebMethod
	public RuleDTO getRule(Long id){
		return this.ruleRepositoryService.getAcceptedRuleDTO(id);
	}

}
