package de.neusta.imonitor.knowledgeserver.webservice;

/*
 * #%L
 * knowledgeServer
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.neusta.imonitor.knowledgeserver.ontology.OntologyProvider;
import de.neusta.imonitor.ruleparser.independencychecker.IndependentRuleChecker;
import de.neusta.imonitor.ruleparser.rules.Rule;
import de.neusta.imonitor.ruleparser.rules.RuleParser;

@Service
public class RuleValidationService {
	
	private final static Logger LOGGER = LoggerFactory.getLogger(RuleValidationService.class);

	@Autowired
	private RuleParser ruleParser;
	
	@Autowired
	private OntologyProvider ontologyProvider;
	
	@Autowired
	private IndependentRuleChecker independentRuleChecker;

	/**
	 * Validates the given xml. Checks that no individual knowledge is present in condition of the rule.
	 * 
	 * @param xml
	 *            The xml to validate.
	 * @return true on success, false on failure.
	 */
	public boolean validate(String xml) {
		try {
			
			Rule rule = ruleParser.parseRule(xml);
			if (rule == null){
				return false;
			}
			return independentRuleChecker.isIndependent(rule);
			
		} catch (Exception e) {
			LOGGER.error("Exception while parsing xml.", e);
			return false;
		}
	}
}
