package de.neusta.imonitor.knowledgeserver.controller;

/*
 * #%L
 * knowledgeServer
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import static org.junit.Assert.*;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.runners.MockitoJUnitRunner;

import de.neusta.imonitor.knowledgeserver.persistence.service.RuleRepositoryService;
import de.neusta.imonitor.knowledgeserver.webservice.dto.PageDTO;

@RunWith(MockitoJUnitRunner.class)
public class RuleControllerTest {

	
	@InjectMocks
	private RuleController ruleController;
	
	@Mock
	private RuleRepositoryService ruleRepositoryService;
	
	private PageDTO<Rule> pageDTO = new PageDTO<Rule>(null, 0, 2);
	
	
	@Before
	public void setup(){
		Mockito.when(ruleRepositoryService.getRulesPage(0)).thenReturn(pageDTO);
		ruleController.init();
	}
	
	@Test
		public void testIsNextPageAvailableFalse() throws Exception {
			this.pageDTO.setNumber(1);
			boolean nextPage = ruleController.isNextPageAvailable();
			
			assertFalse(nextPage);
		}

	@Test
		public void testIsNextPageAvailableTrue() throws Exception {
			this.pageDTO.setNumber(0);
			
			boolean nextPage = ruleController.isNextPageAvailable();
			
			assertTrue(nextPage);
		}
	
	@Test
		public void testIsPreviousPageAvailableFalse() throws Exception {
			this.pageDTO.setNumber(0);
			
			boolean prevPage = ruleController.isPreviousPageAvailable();
			
			assertFalse(prevPage);
		}

	@Test
		public void testIsPreviousPageAvailableTrue() throws Exception {
			this.pageDTO.setNumber(1);
			
			boolean prevPage = ruleController.isPreviousPageAvailable();
			
			assertTrue(prevPage);
		}

	@Test
	public void testNextPage() throws Exception {
		this.ruleController.nextPage();
		
		assertEquals(1, this.ruleController.getPage());
	}

	@Test
	public void testPreviousPage() throws Exception {
		this.ruleController.nextPage();

		this.ruleController.previousPage();
		
		assertEquals(0, this.ruleController.getPage());
	}

	
}
