package de.neusta.imonitor.knowledgeserver.webservice;

/*
 * #%L
 * knowledgeServer
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import static org.junit.Assert.*;

import java.util.ArrayList;
import java.util.List;

import org.junit.Before;
import org.junit.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;

import de.neusta.imonitor.knowledgeserver.converter.RuleDTOToRuleEntityConverter;
import de.neusta.imonitor.knowledgeserver.persistence.entities.RuleEntity;
import de.neusta.imonitor.knowledgeserver.persistence.service.RuleRepositoryService;
import de.neusta.imonitor.knowledgeserver.webservice.dto.RuleDTO;


public class RuleServiceTest {
	
	@Mock
	private RuleRepositoryService ruleRepositoryService;
	
	@Mock
	private RuleDTOToRuleEntityConverter ruleDTOToRuleEntityConverter;
	
	@Mock
	private RuleValidationService ruleValidationService;

	
	@InjectMocks
	private RuleService ruleService;
	
	@Before
	public void setup(){
		MockitoAnnotations.initMocks(this);
	}
	
	@Test
	public void testCommitRuleStoresRuleToDatabase() throws Exception {
		RuleDTO commitRuleDTO = new RuleDTO();
		commitRuleDTO.setName("testname");
		commitRuleDTO.setXml("xml");
		RuleEntity ruleEntity = new RuleEntity();
		Mockito.when(this.ruleDTOToRuleEntityConverter.convert(Mockito.any(RuleDTO.class))).thenReturn(ruleEntity);
		Mockito.when(this.ruleRepositoryService.saveRule(Mockito.any(RuleEntity.class))).thenReturn(ruleEntity);
		Mockito.when(this.ruleValidationService.validate(Mockito.anyString())).thenReturn(true);
		
		boolean commitRule = this.ruleService.commitRule(commitRuleDTO);
		
		assertTrue(commitRule);
		Mockito.verify(this.ruleRepositoryService).saveRule(Mockito.any(RuleEntity.class));
	}
	
	@Test
	public void testCommitRuleDoesNotStoreInvalidRule() throws Exception {
		RuleDTO commitRuleDTO = new RuleDTO();
		commitRuleDTO.setName("testname");
		commitRuleDTO.setXml("xml");
		RuleEntity ruleEntity = new RuleEntity();
		Mockito.when(this.ruleDTOToRuleEntityConverter.convert(Mockito.any(RuleDTO.class))).thenReturn(ruleEntity);
		Mockito.when(this.ruleRepositoryService.saveRule(Mockito.any(RuleEntity.class))).thenReturn(ruleEntity);
		Mockito.when(this.ruleValidationService.validate(Mockito.anyString())).thenReturn(false);
		
		boolean commitRule = this.ruleService.commitRule(commitRuleDTO);
		
		assertFalse(commitRule);
		Mockito.verify(this.ruleRepositoryService, Mockito.times(0)).saveRule(Mockito.any(RuleEntity.class));
	}
	
	@Test
	public void testReturnsFalseOnFailure() throws Exception {
		Mockito.when(this.ruleDTOToRuleEntityConverter.convert(Mockito.any(RuleDTO.class))).thenReturn(new RuleEntity());
		Mockito.when(this.ruleRepositoryService.saveRule(Mockito.any(RuleEntity.class))).thenReturn(null);
		
		boolean commitRule = this.ruleService.commitRule(new RuleDTO());
		
		assertFalse(commitRule);
	}

	@Test
	public void testGetRules() throws Exception {
		List<RuleDTO> acceptedRules = new ArrayList<RuleDTO>();
		Mockito.when(this.ruleRepositoryService.getAcceptedRuleDTOs()).thenReturn(acceptedRules);
		
		List<RuleDTO> rules = this.ruleService.getRules();
		
		assertEquals(acceptedRules, rules);
	}
	

}
