package de.neusta.imonitor.knowledgeserver.webservice;

/*
 * #%L
 * knowledgeServer
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.runners.MockitoJUnitRunner;

import de.neusta.imonitor.knowledgeserver.ontology.OntologyProvider;
import de.neusta.imonitor.ruleparser.independencychecker.IndependentRuleChecker;
import de.neusta.imonitor.ruleparser.rules.Rule;
import de.neusta.imonitor.ruleparser.rules.RuleParser;

@RunWith(MockitoJUnitRunner.class)
public class RuleValidationServiceTest {

	@Mock
	private RuleParser ruleParser;
	
	@Mock
	private IndependentRuleChecker independentRuleChecker;
	
	@Mock
	private OntologyProvider ontologyProvider;

	@InjectMocks
	private RuleValidationService ruleValidator;


	@Test
	public void testValidateFalseOnEmptyString() throws Exception {
		boolean result = this.ruleValidator.validate("");

		assertFalse(result);
	}

	@Test
	public void testValidateFalseForInvalidXml() throws Exception {
		boolean result = this.ruleValidator.validate("abcd");

		assertFalse(result);
	}

	@Test
	public void testValidateTrueForValidXml() throws Exception {
		String ruleXml = "<rule></rule>";
		Mockito.when(ruleParser.parseRule(ruleXml)).thenReturn(new Rule());
		Mockito.when(this.independentRuleChecker.isIndependent(Mockito.any(Rule.class))).thenReturn(true);
		
		boolean valid = this.ruleValidator.validate(ruleXml);
		
		assertTrue(valid);
	}
	
	
	@Test
	public void testValidateFalseForRuleWithIndividual() throws Exception {
		Mockito.when(this.ruleParser.parseRule(Mockito.anyString())).thenReturn(new Rule());
		Mockito.when(independentRuleChecker.isIndependent(Mockito.any(Rule.class))).thenReturn(true);
				
		boolean valid = this.ruleValidator.validate("<rule></rule>");
		
		assertTrue(valid);
		Mockito.verify(this.independentRuleChecker).isIndependent(Mockito.any(Rule.class));
	}


}
