package de.neusta.imonitor.ontology;

/*
 * #%L
 * ontology
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import java.util.List;

import org.junit.Test;

import com.hp.hpl.jena.ontology.Individual;
import com.hp.hpl.jena.ontology.OntClass;
import com.hp.hpl.jena.query.QuerySolution;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.rdf.model.Literal;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.RDFNode;

import de.neusta.imonitor.ontology.tests.TestHelper;
import static org.hamcrest.CoreMatchers.is;
import static org.hamcrest.CoreMatchers.notNullValue;
import static org.hamcrest.CoreMatchers.nullValue;

import static org.junit.Assert.*;

public class OntologyTest {
	@Test
	public void testOntologyCreateInstance() {
		Ontology ontology = TestHelper.loadTestOntology();
		assertNotNull(ontology);
	}

	@Test
	public void testOntologyAsk() throws Exception {
		Ontology ontology = TestHelper.loadTestOntology();
		boolean result = ontology
				.ask("PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>"
						+ "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>"
						+ "PREFIX imonitor: <http://www.imonitor-project.com/ontology#>"
						+ "ASK  { imonitor:WebServer rdfs:subClassOf imonitor:Server; }");
		assertTrue(result);
	}

	@Test
	public void testOntologyCreateIndividual() {
		Ontology ontology = TestHelper.loadTestOntology();
		String individualUri = "imonitor:testIndividual";
		
		Individual individual = ontology.createIndividual(individualUri,
				"imonitor:WebServer");
		
		assertNotNull(individual);
		assertEquals("http://www.imonitor-project.com/ontology#testIndividual", individual.getURI());
	}
	
	@Test
	public void testOntologyCreateIndividualRequestWithSparql() throws Exception {
		Ontology ontology = TestHelper.loadTestOntology();
		String individualUri = "imonitor:testIndividual";
		
		ontology.createIndividual(individualUri, "imonitor:WebServer");
		
		boolean sparqlResult = ontology.ask("PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>"
				+ "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>"
				+ "PREFIX imonitor: <http://www.imonitor-project.com/ontology#>"
				+ "ASK  { imonitor:testIndividual a imonitor:WebServer. }");
		assertThat(sparqlResult, is(true));
	}


	@Test
	public void testOntologySelectAllServers() {
		Ontology ontology = TestHelper.loadTestOntology();
		ResultSet result = ontology
				.select("PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>"
						+ "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>"
						+ "PREFIX imonitor: <http://www.imonitor-project.com/ontology#>"
						+ "SELECT ?x { ?x rdfs:subClassOf imonitor:Server; }");
		
		List<String> variables = result.getResultVars();
		
		assertEquals(1, variables.size());
		assertEquals("x", variables.get(0));
		assertEquals(true, result.hasNext());
	}

	@Test
	public void testOntologyGetConcept() {
		Ontology ontology = TestHelper.loadTestOntology();
		OntClass concept = ontology
				.getConcept("http://www.imonitor-project.com/ontology#Server");
		assertNotNull(concept);
		assertEquals("Server", concept.getLocalName());
	}

	@Test
	public void testOntologyGetIndividual() {
		Ontology ontology = TestHelper.loadTestOntology();
		Individual individual = ontology
				.getIndividual("http://www.imonitor-project.com/ontology#EmptyRecycleBin");
		assertNotNull(individual);
		assertEquals("EmptyRecycleBin", individual.getLocalName());
	}

	@Test
	public void testGetSparqlPrefixes() {
		Ontology ontology = TestHelper.loadTestOntology();
		
		String result = ontology.getSparqlPrefixes();
		
		assertNotNull(ontology.getSparqlPrefixes());
		assertTrue(result.contains("xsd: <http://www.w3.org/2001/XMLSchema#>"));
	}

	@Test
	public void testUpdate() throws Exception {
		Ontology ontology = TestHelper.loadTestOntology();
		ontology.update("PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>"
				+ "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>"
				+ "PREFIX imonitor: <http://www.imonitor-project.com/ontology#>"
				+ "INSERT  { imonitor:myWebserver rdfs:subClassOf imonitor:WebServer } WHERE {}");

		boolean result = ontology
				.ask("PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>"
						+ "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>"
						+ "PREFIX imonitor: <http://www.imonitor-project.com/ontology#>"
						+ "ASK  { imonitor:myWebserver rdfs:subClassOf imonitor:WebServer. }");
		assertTrue(result);
	}
	
	@Test
	public void testUpdateAndCount() throws Exception {
		Ontology ontology = TestHelper.loadTestOntology();
		ontology.update("PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>"
				+ "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>"
				+ "PREFIX imonitor: <http://www.imonitor-project.com/ontology#>"
				+ "INSERT  { imonitor:myIncident rdfs:subClassOf imonitor:Correlation.} WHERE {}");
		ontology.update("PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>"
				+ "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>"
				+ "PREFIX imonitor: <http://www.imonitor-project.com/ontology#>"
				+ "INSERT  { imonitor:myIncident2 rdfs:subClassOf imonitor:Correlation } WHERE {}");

		ontology.select("PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>"
				+ "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>"
				+ "PREFIX imonitor: <http://www.imonitor-project.com/ontology#>"
				+"SELECT ?x (COUNT(?x) AS ?num) WHERE { ?x rdfs:subClassOf imonitor:Correlation; } HAVING (?num > 3)");
		
		boolean result = ontology
				.ask("PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>"
						+ "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>"
						+ "PREFIX imonitor: <http://www.imonitor-project.com/ontology#>"
						+ "ASK WHERE { SELECT (COUNT(?x) AS ?num) WHERE { ?x rdfs:subClassOf imonitor:Correlation; } HAVING (?num > 3) }");
		assertTrue(result);
	}
	
	
	@Test
	public void testUpdateWithTimestamp() throws Exception {
		Ontology ontology = TestHelper.loadTestOntology();
		ontology.update("PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>"
				+ "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>"
				+ "PREFIX imonitor: <http://www.imonitor-project.com/ontology#>"
				+ "PREFIX xsd: <http://www.w3.org/2001/XMLSchema#>"
				+ "INSERT  { imonitor:myIncident rdfs:subClassOf imonitor:Correlation. }"
				+ "WHERE {}");
		ontology.update("PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>"
				+ "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>"
				+ "PREFIX imonitor: <http://www.imonitor-project.com/ontology#>"
				+ "PREFIX xsd: <http://www.w3.org/2001/XMLSchema#>"
				+ "INSERT DATA { imonitor:myIncident imonitor:createDatetime \"1948-01-01T00:00:00-02:00\"^^xsd:dateTime. }");
		ResultSet select = ontology.select("PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>"
				+ "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>"
				+ "PREFIX xsd: <http://www.w3.org/2001/XMLSchema#>"
				+ "PREFIX imonitor: <http://www.imonitor-project.com/ontology#>"
				+ "SELECT ?x { imonitor:myIncident imonitor:createDatetime ?x. }");
		
		assertTrue(select.hasNext());
		QuerySolution solution = select.next();
		RDFNode timeStamp = solution.get("x");
		Literal resultLiteral = timeStamp.asLiteral();
		String resultString = resultLiteral.getString();
		assertEquals("1948-01-01T00:00:00-02:00", resultString);
	}
	
	@Test
	public void testGetProperty() throws Exception {
		Ontology ontology = TestHelper.loadTestOntology();
		
		Property result = ontology.getProperty("imonitor:childOf");
		
		assertThat(result, is(notNullValue()));
	}

	@Test
	public void testCreateRelation() throws Exception {
		Ontology ontology = TestHelper.loadTestOntology();
		
		ontology.createRelation("imonitor:operation1", "imonitor:operation2", "imonitor:parentOf");
		
		Individual sourceIndividual = ontology.getIndividual("imonitor:operation1");
		Property parentRelation = ontology.getProperty("imonitor:parentOf");
		assertThat(sourceIndividual.hasProperty(parentRelation), is(true));
	}

	@Test
	public void testAddDataProperty() throws Exception {
		Ontology ontology = TestHelper.loadTestOntology();
		
		boolean result = ontology.addDataProperty("imonitor:operation1", "imonitor:hasValue", "val");
		
		assertThat(result, is(true));
		Individual individual = ontology.getIndividual("imonitor:operation1");
		Property hasValueRelation = ontology.getProperty("imonitor:hasValue");
		assertThat(individual.hasProperty(hasValueRelation), is(true));
	}

	@Test
	public void testDeleteIndividual() throws Exception {
		Ontology ontology = TestHelper.loadTestOntology();
		
		ontology.deleteIndividual("imonitor:operation1");
		
		Individual result = ontology.getIndividual("imonitor:operation1");
		assertThat(result, is(nullValue()));
	}
	
	@Test
	public void testExample() throws Exception {
		Ontology ontology = TestHelper.loadTestOntology();
		
		ontology.update("PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>PREFIX owl: <http://www.w3.org/2002/07/owl#>PREFIX xsd: <http://www.w3.org/2001/XMLSchema#>PREFIX imonitor: <http://www.imonitor-project.com/ontology#>INSERT DATA { imonitor:hostDownaf9d8fb1-b1c2-460a-8251-cf7cc0e3a846 imonitor:hostaddress \"imonitor:10.240.0\". }");
	}
	
}
