package de.neusta.imonitor.ontology.similarity;

/*
 * #%L
 * ontology
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import static org.junit.Assert.assertTrue;

import org.junit.Test;

import com.hp.hpl.jena.ontology.Individual;
import com.hp.hpl.jena.ontology.OntClass;

import de.neusta.imonitor.ontology.Ontology;
import de.neusta.imonitor.ontology.tests.TestHelper;


public class SimilarityFunctionTest {

	@Test
	public void testSimilarityFunctionWithTwoConcepts(){
		ISimilarityFunction similarityFunction = new SimilarityFunction();
		Ontology ontology = TestHelper.loadTestOntology();
		OntClass serverConcept = ontology.getConcept("http://www.imonitor-project.com/ontology#Server");
		OntClass webServerConcept = ontology.getConcept("http://www.imonitor-project.com/ontology#WebServer");
		float similarity = similarityFunction.determineSimilarity(serverConcept, webServerConcept);
		assertTrue(Math.abs(0.5 - similarity) < 0.001);
	}
	
	@Test
	public void testSimilarityFunctionWithTwoIndividuals(){
		ISimilarityFunction similarityFunction = new SimilarityFunction();
		Ontology ontology = TestHelper.loadTestOntology();
		Individual individualIncreaseHardDiskSize = ontology.getIndividual("http://www.imonitor-project.com/ontology#IncreaseHarddiskSize");
		Individual individualEmptyRecycleBin = ontology.getIndividual("http://www.imonitor-project.com/ontology#EmptyRecycleBin");
		float similarity = similarityFunction.determineSimilarity(individualIncreaseHardDiskSize, individualEmptyRecycleBin);
		assertTrue(Math.abs(0.5 - similarity) < 0.001);
	}

	/**
	 * The individual is an individual of the given concept.
	 */
	@Test
	public void testSimilarityFunctionWithOneIndividualAndOneConcept(){
		ISimilarityFunction similarityFunction = new SimilarityFunction();
		Ontology ontology = TestHelper.loadTestOntology();
		Individual insufficientHarddiskSpaceLeftIndividual = ontology.getIndividual("http://www.imonitor-project.com/ontology#InsufficientHarddiskSpaceLeftOnDevice");
		OntClass incidentConcept = ontology.getConcept("http://www.imonitor-project.com/ontology#HarddiskIncident");
		float similarity = similarityFunction.determineSimilarity(insufficientHarddiskSpaceLeftIndividual, incidentConcept);
		assertTrue(Math.abs(1 - similarity) < 0.001);
	}

}
