package de.neusta.imonitor.rulemanager;

/*
 * #%L
 * rulemanager
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import org.apache.log4j.PropertyConfigurator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.neusta.imonitor.ontology.Ontology;
import de.neusta.imonitor.ontology.configs.OntologyConfig;
import de.neusta.imonitor.rulemanager.model.CommandLineArguments;
import de.neusta.imonitor.rulemanager.ruleloader.RuleLoader;
import de.neusta.imonitor.rulemanager.services.CommandLineProcessingService;
import de.neusta.imonitor.rulewebservice.configs.WebServiceConfig;
import de.neusta.imonitor.rulewebservice.services.OntologyInitService;

@Service
public class RuleManagerStartup {

	private final static Logger LOGGER = LoggerFactory
			.getLogger(RuleManagerStartup.class);

	@Autowired
	private OntologyConfig ontologyConfig;
	@Autowired
	private OntologyInitService ontologyInitService;
	@Autowired
	private RuleLoader ruleLoader;
	@Autowired
	private Ontology ontology;
	@Autowired
	private CommandLineProcessingService commandLineProcessingService;
	@Autowired
	private WebServiceConfig webServiceConfig;

	public void start(String[] args) throws Exception {
		configureLogging();
		checkConfig();
		this.ontologyInitService.loadAndInitOntology();
		this.ruleLoader.loadRulesFromDatabase(ontology);
		CommandLineArguments commandLineArguments = new CommandLineArguments(args);
		commandLineProcessingService.process(commandLineArguments);
	}

	private void checkConfig() {
		if (webServiceConfig.getWsdlUrl() == null) {
			LOGGER.warn("There is no url specified for the web service. Please check the 'webservice.properties'.");
		}
		
	}

	private void configureLogging() {
		// configure log4j
		PropertyConfigurator.configure("../configs/log4j.properties");
	}

}
