package de.neusta.imonitor.rulemanager.model;

/*
 * #%L
 * rulemanager
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import org.kohsuke.args4j.CmdLineException;
import org.kohsuke.args4j.CmdLineParser;
import org.kohsuke.args4j.Option;

public class CommandLineArguments {
	@Option(name = "-l", aliases = { "--list" }, required = false, usage = "list all locally installed rules")
	private boolean list;

	@Option(name = "-d", aliases = { "--delete" }, required = false, usage = "deletes an installed rule with the given id")
	private Long deleteId;

	@Option(name = "-a", aliases = { "--add" }, required = false, usage = "add a rule to the database")
	private String addRuleFile;

	@Option(name = "-ls", aliases = { "--listserver" }, required = false, usage = "lists all rules on the server")
	private boolean listServerRules;

	@Option(name = "-c", aliases = { "--commit" }, required = false, usage = "commits a rule to the server")
	private Long commitId;

	@Option(name = "-f", aliases = { "--fetch" }, required = false, usage = "fetches a rule from the server")
	private Long fetchId;

	@Option(name = "-n", aliases = { "--name" }, required = false, usage = "the name of the rule")
	private String ruleName;
	
	@Option(name = "-ws", aliases = {"--webservice"}, required = false, usage = "the url to the webservice")
	private String webserviceUrl;
	
	@Option(name = "-e", aliases = {"--export"}, required = false, usage = "exports the xml to disk, uses the rule id as filename")
	private Long exportId;
	
	private CmdLineParser commandLineParser;

	

	public CommandLineArguments(String... args) {
		commandLineParser = new CmdLineParser(this);
		try {
			if (args != null) {
				commandLineParser.parseArgument(args);
			} else {
				commandLineParser.printUsage(System.out);
			}
		} catch (CmdLineException e) {
			System.out.println("Found at least one illegal argument.");
			commandLineParser.printUsage(System.out);
		}

	}

	public boolean isList() {
		return list;
	}

	public void setList(boolean list) {
		this.list = list;
	}

	public Long getDeleteId() {
		return deleteId;
	}

	public void setDeleteId(Long deleteId) {
		this.deleteId = deleteId;
	}

	public Long getCommitId() {
		return commitId;
	}

	public void setCommitId(Long commitId) {
		this.commitId = commitId;
	}

	public Long getFetchId() {
		return fetchId;
	}

	public void setFetchId(Long fetchId) {
		this.fetchId = fetchId;
	}

	public String getAddRuleFile() {
		return addRuleFile;
	}

	public void setAddRuleFile(String addRuleFile) {
		this.addRuleFile = addRuleFile;
	}

	public String getRuleName() {
		return ruleName;
	}

	public void setRuleName(String ruleName) {
		this.ruleName = ruleName;
	}

	public CmdLineParser getCommandLineParser() {
		return commandLineParser;
	}

	public String getWebserviceUrl() {
		return webserviceUrl;
	}

	public Long getExportId() {
		return exportId;
	}

	public boolean isListServerRules() {
		return listServerRules;
	}
}
