package de.neusta.imonitor.rulemanager.services;

/*
 * #%L
 * rulemanager
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */

import javax.transaction.Transactional;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.neusta.imonitor.correlation.persistency.entities.RuleEntity;
import de.neusta.imonitor.ontology.Ontology;
import de.neusta.imonitor.rulemanager.persistency.repositories.RuleRepository;
import de.neusta.imonitor.ruleparser.rules.Rule;
import de.neusta.imonitor.ruleparser.rules.RuleParser;

/**
 * Adds a rule from a xml file to the rule repository.
 *
 */
@Service
public class AddRuleService {

	private static final Logger LOGGER = LoggerFactory.getLogger(AddRuleService.class);
	
	@Autowired
	private Ontology ontology;
	@Autowired
	private RuleParser ruleParser;
	@Autowired
	private ReadTextService readTextService;
	@Autowired
	private RuleRepository ruleRepository;

	@Transactional
	public void add(String ruleFile, String ruleName) throws Exception {
		if(StringUtils.isBlank(ruleName))
		{
			LOGGER.error("The rulename was empty. Rule was not saved.");
			throw new IllegalArgumentException("ruleName cannot be empty.");
		}
		if(StringUtils.isBlank(ruleFile))
		{
			LOGGER.error("The rulefilename was empty. Rule was not saved.");
			throw new IllegalArgumentException("ruleFile name cannot be empty.");
		}
		String fileContent = readTextService.read(ruleFile);
		Rule parsedRule = ruleParser.parseRule(fileContent);
		if (parsedRule == null) {
			LOGGER.error("Could not parse rule xml.");
			return;
		}
		RuleEntity entity = new RuleEntity();
		entity.setRule(fileContent);		
		entity.setName(ruleName);
		ruleRepository.save(entity);
		LOGGER.info("Rule saved. Id: " + entity.getId());

	}

}
