package de.neusta.imonitor.rulemanager.services;

/*
 * #%L
 * rulemanager
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import java.net.MalformedURLException;
import java.net.URL;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.neusta.imonitor.rulemanager.model.CommandLineArguments;
import de.neusta.imonitor.rulewebservice.RuleWebServiceApi;
import de.neusta.imonitor.rulewebservice.configs.WebServiceConfig;

@Service
public class CommandLineProcessingService {
	
	private static final Logger LOGGER = LoggerFactory.getLogger(CommandLineProcessingService.class);
	
	@Autowired
	private ListRulesService listRulesService;
	@Autowired
	private DeleteRuleService deleteRuleService;
	@Autowired
	private CommitRuleService commitRuleService;
	@Autowired
	private FetchRuleService fetchRuleService;
	@Autowired
	private AddRuleService addRuleService;
	@Autowired
	private ExportRuleService exportRuleService;
	@Autowired
	private ListServerRulesService listServerRulesService;
	@Autowired
	private WebServiceConfig webServiceConfig;
	@Autowired
	private RuleWebServiceApi ruleWebServiceApi;

	public void process(CommandLineArguments commandLineArguments) throws Exception {
		try {
			if (commandLineArguments.isList()) {
				listRulesService.list();
			} else if (commandLineArguments.getDeleteId() != null) {
				deleteRuleService.delete(commandLineArguments.getDeleteId());
			} else if (commandLineArguments.getAddRuleFile() != null) {
				addRuleService.add(commandLineArguments.getAddRuleFile(), commandLineArguments.getRuleName());
			} else if (commandLineArguments.getCommitId() != null) {
				prepareWebservice(commandLineArguments);
				commitRuleService.commit(commandLineArguments.getCommitId(), commandLineArguments);
			} else if (commandLineArguments.getFetchId() != null) {
				prepareWebservice(commandLineArguments);
				fetchRuleService.fetch(commandLineArguments.getFetchId(), commandLineArguments);
			} else if(commandLineArguments.getExportId() != null) {
				exportRuleService.export(commandLineArguments.getExportId());
			}  else if (commandLineArguments.isListServerRules()) {
				prepareWebservice(commandLineArguments);
				listServerRulesService.list(commandLineArguments);
			} else {
				System.out.println("ERROR: Unknown argument.");
				commandLineArguments.getCommandLineParser().printUsage(System.out);
			}
		} catch (Exception e) {
			LOGGER.error("Exception while processing command.", e);
		}
	}
	
	private void prepareWebservice(CommandLineArguments commandLineArguments) throws MalformedURLException {
		URL url;
		if (!StringUtils.isBlank(commandLineArguments.getWebserviceUrl())) {
			LOGGER.info(String.format("Using webservice url from command line argument: '%s'", commandLineArguments.getWebserviceUrl()));
			url = new URL(commandLineArguments.getWebserviceUrl());
		} else {
			LOGGER.info(String.format("Using webservice url from config file: '%s'", webServiceConfig.getWsdlUrl()));
			url = new URL(webServiceConfig.getWsdlUrl());
		}
		ruleWebServiceApi.setWebserviceUrl(url);
		
	}

}
