package de.neusta.imonitor.rulemanager.services;

/*
 * #%L
 * rulemanager
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import javax.transaction.Transactional;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.neusta.imonitor.correlation.persistency.entities.RuleEntity;
import de.neusta.imonitor.knowledgeserver.webservice.RuleDTO;
import de.neusta.imonitor.rulemanager.model.CommandLineArguments;
import de.neusta.imonitor.rulemanager.persistency.repositories.RuleRepository;
import de.neusta.imonitor.rulemanager.transformer.RuleTransformation;
import de.neusta.imonitor.rulewebservice.RuleWebServiceApi;

@Service
public class CommitRuleService {

	private static final Logger LOGGER = LoggerFactory.getLogger(CommitRuleService.class);
	
	@Autowired
	private RuleWebServiceApi ruleWebServiceApi;
	
	@Autowired
	private RuleRepository ruleRepository;
	
	@Transactional
	public boolean commit(Long commitId, CommandLineArguments commandLineArguments) throws Exception{
		RuleEntity rule = ruleRepository.findOne(commitId);
		if(rule == null) {
			LOGGER.error("The rule with id: " + commitId + " could not be loaded from database.");
			throw new Exception("Rule with ID: " + commitId + " not found in database.");
		}
		RuleDTO convertedRule = RuleTransformation.convertToDTO(rule);
		if (!StringUtils.isBlank(commandLineArguments.getRuleName())){
			System.out.println(String.format("Adjusting rule name to : '%s'", commandLineArguments.getRuleName()));
			convertedRule.setName(commandLineArguments.getRuleName());
		}
		boolean result = ruleWebServiceApi.commitRule(convertedRule);
		if(result) {
			LOGGER.info("The rule with id: " + commitId + " was successfully commited to webservice.");
		} else {
			LOGGER.error("The rule with id: " + commitId + " could not be commited to webservice. The webservice has not accepted the rule.");
		}
		return result;
	}

}
