package de.neusta.imonitor.rulemanager.services;

/*
 * #%L
 * rulemanager
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import java.net.MalformedURLException;

import javax.transaction.Transactional;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.neusta.imonitor.correlation.persistency.entities.RuleEntity;
import de.neusta.imonitor.knowledgeserver.webservice.RuleDTO;
import de.neusta.imonitor.rulemanager.model.CommandLineArguments;
import de.neusta.imonitor.rulemanager.persistency.repositories.RuleRepository;
import de.neusta.imonitor.rulemanager.transformer.RuleTransformation;
import de.neusta.imonitor.rulewebservice.RuleWebServiceApi;

@Service
public class FetchRuleService {
	
	private static final Logger LOGGER = LoggerFactory.getLogger(DeleteRuleService.class);
	
	@Autowired
	private RuleWebServiceApi ruleWebServiceApi;
	
	@Autowired
	private RuleRepository ruleRepository;

	@Transactional
	public void fetch(Long fetchId, CommandLineArguments commandLineArguments) throws MalformedURLException{
		RuleDTO dto = ruleWebServiceApi.getRule(fetchId);
						
		if(dto != null)
		{
			RuleEntity entity = RuleTransformation.convertToEntity(dto);
			ruleRepository.save(entity);
			LOGGER.info("Rule fetched.");
		}
		else
		{
			LOGGER.error("Rule cannot be fetched because rule with id: " + fetchId + " could not be returned by service.");
		}
		
	}

}
