package de.neusta.imonitor.rulemanager.services;

/*
 * #%L
 * rulemanager
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import static org.junit.Assert.*;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.runners.MockitoJUnitRunner;

import de.neusta.imonitor.correlation.persistency.entities.RuleEntity;
import de.neusta.imonitor.rulemanager.persistency.repositories.RuleRepository;
import de.neusta.imonitor.ruleparser.rules.Rule;
import de.neusta.imonitor.ruleparser.rules.RuleParser;

@RunWith(MockitoJUnitRunner.class)
public class AddRuleServiceTest {

	@InjectMocks
	private AddRuleService addRuleService;

	@Mock
	private ReadTextService readTextService;

	@Mock
	private RuleParser ruleParser;

	@Mock
	private RuleRepository ruleRepository;

	@Captor
	private ArgumentCaptor<RuleEntity> ruleEntityCaptor;

	@Test
	public void testAddReadsFile() throws Exception {
		addRuleService.add("file.xml", "test");

		Mockito.verify(readTextService).read("file.xml");
	}

	@Test
	public void testAddCreatesDataBaseEntry() throws Exception {
		String content = "testcontent";
		String ruleFile = "file.xml";
		Mockito.when(readTextService.read(ruleFile)).thenReturn(content);
		Rule parsedRule = new Rule();
		Mockito.when(ruleParser.parseRule(content)).thenReturn(parsedRule);

		addRuleService.add(ruleFile, "test");

		Mockito.verify(ruleRepository).save(Mockito.any(RuleEntity.class));
	}

	@Test
	public void testAddInsertsFileContentToDatabaseEntity() throws Exception {
		String ruleFile = "file.xml";
		String content = "testcontent";
		Mockito.when(readTextService.read(ruleFile)).thenReturn(content);
		Rule parsedRule = new Rule();
		Mockito.when(ruleParser.parseRule(content)).thenReturn(parsedRule);

		addRuleService.add(ruleFile, "test");

		Mockito.verify(ruleRepository).save(ruleEntityCaptor.capture());
		RuleEntity capturedRuleEntity = ruleEntityCaptor.getValue();
		assertEquals(content, capturedRuleEntity.getRule());
		assertEquals("test", capturedRuleEntity.getName());
		Mockito.verify(ruleParser).parseRule(content);
	}
	
	@Test
	public void testAddDoesNotInsertsFileContentToDatabaseIfSyntaxError() throws Exception {
		String ruleFile = "file.xml";
		String content = "testcontent";
		Mockito.when(readTextService.read(ruleFile)).thenReturn(content);
		Mockito.when(ruleParser.parseRule(content)).thenReturn(null);

		addRuleService.add(ruleFile, "test");

		Mockito.verify(ruleRepository, Mockito.times(0)).save(ruleEntityCaptor.capture());
	}


	@Test(expected = IllegalArgumentException.class)
	public void testAddShouldThrowIllegalArgumentExceptionOnEmptyRuleName() throws Exception {
		addRuleService.add("file.txt", "");
	}

	@Test(expected = IllegalArgumentException.class)
	public void testAddShouldThrowIllegalArgumentExceptionOnNullRuleName() throws Exception {
		addRuleService.add("file.txt", null);
	}

	@Test(expected = IllegalArgumentException.class)
	public void testAddShouldThrowIllegalArgumentExceptionOnEmptyRuleFile() throws Exception {
		addRuleService.add("", "test");
	}

	@Test(expected = IllegalArgumentException.class)
	public void testAddShouldThrowIllegalArgumentExceptionOnNullRuleFile() throws Exception {
		addRuleService.add(null, "test");
	}
}
