package de.neusta.imonitor.rulemanager.services;

/*
 * #%L
 * rulemanager
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.runners.MockitoJUnitRunner;

import de.neusta.imonitor.rulemanager.model.CommandLineArguments;
import de.neusta.imonitor.rulewebservice.RuleWebServiceApi;
import de.neusta.imonitor.rulewebservice.configs.WebServiceConfig;

@RunWith(MockitoJUnitRunner.class)
public class CommandLineProcessingServiceTest {

	@InjectMocks
	private CommandLineProcessingService commandLineProcessingService;
	@Mock
	private ListRulesService listRulesService;
	@Mock
	private DeleteRuleService deleteRuleService;
	@Mock
	private CommitRuleService commitRuleService;
	@Mock
	private FetchRuleService fetchRuleService;
	@Mock
	private AddRuleService addRuleService;
	@Mock
	private ExportRuleService exportRuleService;
	@Mock
	private ListServerRulesService listServerRulesService;
	@Mock
	private RuleWebServiceApi ruleWebServiceApi;
	@Mock
	private WebServiceConfig webServiceConfig;
	
	@Before
	public void setup() {
		Mockito.when(webServiceConfig.getWsdlUrl()).thenReturn("http://example.com");
	}



	@Test()
	public void testProcessUnknownArgument() throws Exception {
		CommandLineArguments commandLineArguments = new CommandLineArguments("");
		commandLineProcessingService.process(commandLineArguments);
	}
	
	@Test
	public void testProcessList() throws Exception {
		CommandLineArguments commandLineArguments = new CommandLineArguments("-l");

		commandLineProcessingService.process(commandLineArguments);

		Mockito.verify(listRulesService).list();
	}

	@Test
	public void testProcessDelete() throws Exception {
		CommandLineArguments commandLineArguments = new CommandLineArguments("-d", "5");

		commandLineProcessingService.process(commandLineArguments);

		Mockito.verify(deleteRuleService).delete(new Long(5));
	}

	@Test
	public void testProcessDeleteNull() throws Exception {
		CommandLineArguments commandLineArguments = new CommandLineArguments("");

		commandLineProcessingService.process(commandLineArguments);

		Mockito.verify(deleteRuleService, Mockito.times(0)).delete(Mockito.any(Long.class));
	}

	@Test
	public void testProcessCommit() throws Exception {
		CommandLineArguments commandLineArguments = new CommandLineArguments("-c", "5");

		commandLineProcessingService.process(commandLineArguments);

		Mockito.verify(commitRuleService).commit(new Long(5), commandLineArguments);
	}

	@Test
	public void testProcessCommitNull() throws Exception {
		CommandLineArguments commandLineArguments = new CommandLineArguments("");

		commandLineProcessingService.process(commandLineArguments);

		Mockito.verify(commitRuleService, Mockito.times(0)).commit(Mockito.any(Long.class), Mockito.eq(commandLineArguments));
	}

	@Test
	public void testProcessFetch() throws Exception {
		CommandLineArguments commandLineArguments = new CommandLineArguments("-f", "5");

		commandLineProcessingService.process(commandLineArguments);

		Mockito.verify(fetchRuleService).fetch(new Long(5), commandLineArguments);
	}

	@Test
	public void testProcessFetchNull() throws Exception {
		CommandLineArguments commandLineArguments = new CommandLineArguments("");

		commandLineProcessingService.process(commandLineArguments);

		Mockito.verify(fetchRuleService, Mockito.times(0)).fetch(Mockito.any(Long.class), Mockito.eq(commandLineArguments));
	}

	@Test
	public void testProcessAdd() throws Exception {
		CommandLineArguments commandLineArguments = new CommandLineArguments("-a", "test.xml");

		commandLineProcessingService.process(commandLineArguments);

		Mockito.verify(addRuleService).add(Mockito.eq("test.xml"), Mockito.anyString());
	}

	@Test
	public void testProcessAddNull() throws Exception {
		CommandLineArguments commandLineArguments = new CommandLineArguments("");

		commandLineProcessingService.process(commandLineArguments);

		Mockito.verify(addRuleService, Mockito.times(0)).add(Mockito.anyString(), Mockito.anyString());
	}

	@Test
	public void testProcessExport() throws Exception {
		CommandLineArguments commandLineArguments = new CommandLineArguments("-e", "005");

		commandLineProcessingService.process(commandLineArguments);

		Mockito.verify(exportRuleService).export(Mockito.eq(new Long(5)));
	}

	@Test
	public void testProcessListServerRules() throws Exception {
		CommandLineArguments commandLineArguments = new CommandLineArguments("-ls");

		commandLineProcessingService.process(commandLineArguments);

		Mockito.verify(listServerRulesService).list(commandLineArguments);
	}

}
