package de.neusta.imonitor.rulemanager.services;

/*
 * #%L
 * rulemanager
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import static org.junit.Assert.*;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.runners.MockitoJUnitRunner;

import de.neusta.imonitor.correlation.persistency.entities.RuleEntity;
import de.neusta.imonitor.knowledgeserver.webservice.RuleDTO;
import de.neusta.imonitor.rulemanager.persistency.repositories.RuleRepository;
import de.neusta.imonitor.rulewebservice.RuleWebServiceApi;

@RunWith(MockitoJUnitRunner.class)
public class FetchRuleServiceTest {

	@InjectMocks
	private FetchRuleService fetchRuleService;
	
	@Mock
	private RuleWebServiceApi mockedRuleWebservice;
	
	@Mock
	private RuleRepository mockedRuleRepository;
	
	@Captor
	private ArgumentCaptor<RuleEntity> ruleEntityCaptor;
	
	@Test
	public void testFetchCallsServiceAndUpdatesDatabase() throws Exception {
		long id = 1;
		RuleDTO dto = new RuleDTO();
		dto.setId(id);
		dto.setName("Testname");
		dto.setXml("Testxml");
		Mockito.when(mockedRuleWebservice.getRule(id)).thenReturn(dto);
				
		fetchRuleService.fetch(id, null);
		
		Mockito.verify(mockedRuleRepository).save(ruleEntityCaptor.capture());
		RuleEntity capturedRuleEntity = ruleEntityCaptor.getValue();
		assertNotEquals((long)dto.getId(), capturedRuleEntity.getId());
		assertEquals(dto.getName(), capturedRuleEntity.getName());
		assertEquals(dto.getXml(), capturedRuleEntity.getRule());
		
	}
}
