package de.neusta.imonitor.rulemanager.transformer;

/*
 * #%L
 * rulemanager
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.runners.MockitoJUnitRunner;

import de.neusta.imonitor.correlation.persistency.entities.RuleEntity;
import de.neusta.imonitor.knowledgeserver.webservice.RuleDTO;
import de.neusta.imonitor.rulemanager.transformer.RuleTransformation;

@RunWith(MockitoJUnitRunner.class)
public class RuleTransformationTest {

	@Test
	public void testConvertToDTOShouldReturnValidDTO() throws Exception {
		RuleEntity rule = new RuleEntity();
		rule.setId(1l);
		rule.setName("test");
		rule.setRule("testrulexml");
		
		RuleDTO result = RuleTransformation.convertToDTO(rule);
		
		Assert.assertNotNull(result);
		Assert.assertEquals(rule.getName(),result.getName());
		Assert.assertEquals((Long)rule.getId(),result.getId());
		Assert.assertEquals(rule.getRule(),result.getXml());
	}
	
	@Test(expected = IllegalArgumentException.class)
	public void testConvertToDTOShouldThrowExceptionOnNullValue() throws Exception {
		RuleTransformation.convertToDTO(null);
	}
	
	@Test
	public void testConvertToEntityShouldReturnValidEntity() throws Exception {
		RuleDTO rule = new RuleDTO();
		rule.setId(1l);
		rule.setName("test");
		rule.setXml("testrulexml");
		
		RuleEntity result = RuleTransformation.convertToEntity(rule);
		
		Assert.assertNotNull(result);
		Assert.assertEquals(rule.getName(),result.getName());
		Assert.assertNotEquals((long)rule.getId(),result.getId());
		Assert.assertEquals(rule.getXml(),result.getRule());
	}
	
	@Test(expected = IllegalArgumentException.class)
	public void testCovertToEntityShouldThrowExceptionOnNullValue() throws Exception {
		
		RuleTransformation.convertToEntity(null);
	}
}
