package de.neusta.imonitor.correlation;

import java.util.List;

import junit.framework.Assert;

import org.junit.Test;

import de.neusta.imonitor.correlation.rules.Rule;
import de.neusta.imonitor.correlation.rules.RuleParser;
import de.neusta.imonitor.ontology.Ontology;

public class RuleParserTests {

	@Test
	public void testCreateRuleParserInstance(){
		RuleParser ruleParser = new RuleParser();
		Assert.assertNotNull(ruleParser);
	}
	
	@Test
	public void testParseRuleFileLoadsXMLFileWithoutException() throws Exception{
		TestHelper.getTestRuleParser();
	}

	@Test
	public void testParseRuleFileLoadsXMLFileWithCondition() throws Exception{
		RuleParser ruleParser = TestHelper.getTestRuleParser();
		List<Rule> rules = ruleParser.getRules();
		Assert.assertEquals(1, rules.size());
		Assert.assertNotNull(rules.get(0).getCondition());
		Assert.assertEquals("ASK { $var1 a Abstract(imonitor:WebServer, imonitor:Server, \"Server\"); }", rules.get(0).getCondition().getQuery());
	}

	@Test
	public void testParseRuleFileLoadsXMLFileWithAction() throws Exception{
		RuleParser ruleParser = TestHelper.getTestRuleParser();
		List<Rule> rules = ruleParser.getRules();
		Assert.assertEquals(1, rules.size());
		Assert.assertNotNull(rules.get(0).getAction());
		Assert.assertEquals(1,rules.get(0).getAction().getIncidents().size());
		Assert.assertFalse(rules.get(0).getAction().getCommand().isEmpty());
		Assert.assertEquals("imonitor:InsufficientHarddiskSpaceLeftOnDevice",rules.get(0).getAction().getIncidents().get(0).getUri());
		Assert.assertEquals("someIncident",rules.get(0).getAction().getIncidents().get(0).getName());
		// check explanations
		Assert.assertEquals(1, rules.get(0).getAction().getExplanations().size());
		Assert.assertEquals("The critical process(es) ?process are dependent on the unreachable host $hostaddress.", rules.get(0).getAction().getExplanations().get(0));
		// check action queries
		Assert.assertEquals(1, rules.get(0).getAction().getQueries().size());
		Assert.assertEquals("SELECT ?process { ?process a imonitor:Process. ?process imonitor:dependsOn $hostaddress. ?process imonitor:hasCriticality imonitor:HIGH_CRITICALITY.}", rules.get(0).getAction().getQueries().get(0));
		// check recommendations
		Assert.assertEquals(1, rules.get(0).getAction().getRecommendations().size());
		Assert.assertEquals("Ensure that the host $hostaddress is running and reachable.", rules.get(0).getAction().getRecommendations().get(0));
		
	}
}
