package de.neusta.imonitor.correlation.explanationprocessor;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import junit.framework.Assert;

import org.junit.Test;

import de.neusta.imonitor.correlation.abstraction.Abstraction;
import de.neusta.imonitor.correlation.rules.Action;
import de.neusta.imonitor.ontology.VariableList;

public class ExplanationProcessorTest {
	
	private static final String TESTEXPLANATION = "testexplanation";

	/**
	 * Tests that a simple explanation is added to the explanation list.
	 */
	@Test
	public void testExplanationIsGenerated(){
		ExplanationProcessor explanationProcessor = new ExplanationProcessor();
		Action action = new Action();
		action.addExplanation(TESTEXPLANATION);
		VariableList variableList = new VariableList();
		HashMap<String, List<String>> actionQueryVariables = new HashMap<String, List<String>>();
		Abstraction abstraction = new Abstraction();
		List<String> testExplanation = explanationProcessor.processExplanations(action, variableList, actionQueryVariables, abstraction);
		Assert.assertNotNull(testExplanation);
		Assert.assertEquals(1, testExplanation.size());
		Assert.assertEquals(TESTEXPLANATION, testExplanation.get(0));
	}
	
	/**
	 * Tests that an input variable is replaced in the explanation.
	 * @throws Exception 
	 */
	@Test
	public void testExplanationUsesInputVariable() throws Exception{
		ExplanationProcessor explanationProcessor = new ExplanationProcessor();
		Action action = new Action();
		action.addExplanation("Explanation variable: $testvar.");
		VariableList variableList = new VariableList();
		variableList.createVariable("testvar", "testval");
		HashMap<String, List<String>> actionQueryVariables = new HashMap<String, List<String>>();
		Abstraction abstraction = new Abstraction();
		List<String> testExplanation = explanationProcessor.processExplanations(action, variableList, actionQueryVariables, abstraction);
		Assert.assertNotNull(testExplanation);
		Assert.assertEquals(1, testExplanation.size());
		Assert.assertEquals("Explanation variable: testval.", testExplanation.get(0));
	}
	
	/**
	 * Tests that query variables are used.
	 * @throws Exception
	 */
	@Test
	public void testExplanationUsesQueryVariable() throws Exception{
		ExplanationProcessor explanationProcessor = new ExplanationProcessor();
		Action action = new Action();
		action.addExplanation("Explanation variable: ?testvar.");
		VariableList variableList = new VariableList();
		HashMap<String, List<String>> actionQueryVariables = new HashMap<String, List<String>>();
		List<String> values = new ArrayList<String>();
		values.add("testval");
		actionQueryVariables.put("testvar", values);
		Abstraction abstraction = new Abstraction();
		List<String> testExplanation = explanationProcessor.processExplanations(action, variableList, actionQueryVariables, abstraction);
		Assert.assertNotNull(testExplanation);
		Assert.assertEquals(1, testExplanation.size());
		Assert.assertEquals("Explanation variable: testval.", testExplanation.get(0));
	}

}
