package de.neusta.imonitor.correlation.queryprocessor;

import java.util.HashMap;
import java.util.List;

import junit.framework.Assert;

import org.junit.Test;

import de.neusta.imonitor.correlation.TestHelper;
import de.neusta.imonitor.correlation.abstraction.Abstraction;
import de.neusta.imonitor.correlation.abstraction.AbstractionVariable;
import de.neusta.imonitor.correlation.rules.Action;
import de.neusta.imonitor.ontology.Ontology;
import de.neusta.imonitor.ontology.VariableList;

public class QueryProcessorTest {
	
	/**
	 * Tests that a query is evaluated on the ontology and returns a value.
	 */
	@Test
	public void testQueryProcessor(){
		QueryProcessor queryProcessor = createTestQueryProcessor();
		Action action = new Action();
		action.addQuery("SELECT ?x { ?x a imonitor:WebServer. }");
		VariableList variableList = new VariableList();
		HashMap<String,List<String>> processQueries = queryProcessor.processQueries(action, variableList, null);
		Assert.assertNotNull(processQueries);
		Assert.assertEquals(1, processQueries.size());
		Assert.assertTrue(processQueries.containsKey("x"));
		Assert.assertTrue(processQueries.get("x").size() > 0);
	}

	private QueryProcessor createTestQueryProcessor() {
		Ontology testOntology = TestHelper.getTestOntology();
		QueryProcessor queryProcessor = new QueryProcessor(testOntology);
		return queryProcessor;
	}
	
	@Test
	public void testQueryProcessorWithAbstractions(){
		QueryProcessor queryProcessor = createTestQueryProcessor();
		Action action = new Action();
		action.addQuery("SELECT ?x { ?x a UseAbstract(imonitor:WebServer, testAbs).}");
		VariableList variableList = new VariableList();
		Abstraction abstraction = new Abstraction();
		AbstractionVariable[] abstractionVariables = new AbstractionVariable[1];
		abstractionVariables[0] = new AbstractionVariable();
		abstractionVariables[0].setVariableName("testAbs");
		abstractionVariables[0].setVariableValue("imonitor:Server");
		abstraction.setAbstractionVariables(abstractionVariables);
		HashMap<String,List<String>> processQueries = queryProcessor.processQueries(action, variableList, abstraction);
		Assert.assertNotNull(processQueries);
		Assert.assertEquals(1, processQueries.size());
		Assert.assertTrue(processQueries.containsKey("x"));
		Assert.assertTrue(processQueries.get("x").size() > 0);
	}

}
