package de.neusta.imonitor.ruleparser.independencychecker;

/*
 * #%L
 * ruleparser
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.neusta.imonitor.ontology.Ontology;

@Service
public class IndependentQueryChecker {

	@Autowired
	private Ontology ontology;

	@Autowired
	private IndependencyExceptions independencyExceptions;

	public boolean isIndependet(String independentQuery) {
		List<String> literals = LiteralParser.getLiteralsFromSparqlQuery(independentQuery);
		boolean result = true;
		for (String literal : literals) {
			if (!independencyExceptions.isException(literal)) {
				this.ontology.readLock();
				try {
					boolean isConcept = ontology.getConcept(literal) != null;
					boolean isProperty = ontology.getProperty(literal) != null;
					result &= isConcept || isProperty;
				} finally {
					this.ontology.releaseLock();
				}
			}
		}
		return result;
	}

}
