package de.neusta.imonitor.ruleparser.independencychecker;

/*
 * #%L
 * ruleparser
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.neusta.imonitor.ruleparser.rules.Action;
import de.neusta.imonitor.ruleparser.rules.Condition;
import de.neusta.imonitor.ruleparser.rules.Rule;

@Service
public class IndependentRuleChecker {
	
	@Autowired
	private IndependentQueryChecker independentQueryChecker;

	public boolean isIndependent(Rule rule) {
		if (!independentCondition(rule)){
			return false;
		}
		if (!independentAction(rule)){
			return false;
		}
		return true;
	}

	private boolean independentAction(Rule rule) {
		Action action = rule.getAction();
		if (action == null){
			return true;
		}
		List<String> queries = action.getQueries();
		for (String query : queries) {
			if (!this.independentQueryChecker.isIndependet(query)){
				return false;
			}
		}
		List<String> updateQueries = action.getUpdateQueries();
		for (String updateQuery : updateQueries) {
			if (!this.independentQueryChecker.isIndependet(updateQuery)){
				return false;
			}
		}
		return true;
	}

	private boolean independentCondition(Rule rule) {
		Condition condition = rule.getCondition();
		if (condition == null){
			return true;
		}
		String query = condition.getQuery();
		return this.independentQueryChecker.isIndependet(query);
	}	
}
