package de.neusta.imonitor.ruleparser.independencychecker;

/*
 * #%L
 * ruleparser
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import static org.hamcrest.CoreMatchers.is;

import static org.junit.Assert.*;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.runners.MockitoJUnitRunner;

import com.hp.hpl.jena.ontology.OntClass;
import com.hp.hpl.jena.rdf.model.Property;

import de.neusta.imonitor.ontology.Ontology;

@RunWith(MockitoJUnitRunner.class)
public class IndependentQueryCheckerTest {

	@InjectMocks
	private IndependentQueryChecker independentQueryChecker;
	@Mock
	private Ontology ontology;
	@Mock
	private OntClass ontClass;
	@Mock
	private IndependencyExceptions independencyExceptions;
	@Mock
	private Property property;

	@Test
	public void testIsIndependentEmptyString() throws Exception {
		boolean independet = this.independentQueryChecker.isIndependet("");

		assertTrue(independet);
	}

	@Test
	public void testIsIndependentFalseWithIndividual() throws Exception {
		Mockito.when(this.ontology.getConcept(Mockito.eq("imonitor:myOpenStackServer"))).thenReturn(null);
		boolean independet = this.independentQueryChecker.isIndependet("ASK { $hostaddress a imonitor:myOpenStackServer }");

		assertFalse(independet);
	}

	@Test
	public void testIsIndependentTrueWithConcept() throws Exception {
		Mockito.when(this.ontology.getConcept(Mockito.eq("imonitor:OpenStackServer"))).thenReturn(this.ontClass);
		boolean independet = this.independentQueryChecker.isIndependet("ASK { $hostaddress a imonitor:OpenStackServer }");

		assertTrue(independet);
	}

	@Test
	public void testIndipendentQueryExample() throws Exception {
		Mockito.when(this.ontology.getConcept(Mockito.eq("imonitor:ServicestateCritical"))).thenReturn(this.ontClass);
		Mockito.when(this.ontology.getConcept(Mockito.eq("imonitor:Service"))).thenReturn(this.ontClass);
		Mockito.when(this.ontology.getConcept(Mockito.eq("imonitor:CriticalityHigh"))).thenReturn(this.ontClass);
		Mockito.when(this.ontology.getProperty(Mockito.eq("imonitor:hasName"))).thenReturn(property);
		Mockito.when(this.ontology.getProperty(Mockito.eq("imonitor:hasCriticality"))).thenReturn(property);
		Mockito.when(this.ontology.getProperty(Mockito.eq("imonitor:affects"))).thenReturn(property);
		
		boolean result = this.independentQueryChecker
				.isIndependet("ASK { ?servicestate imonitor:hasName \"$SERVICESTATE\".				?servicestate a imonitor:ServicestateCritical.				?service a imonitor:Service.				?service imonitor:hasName \"$SERVICEDESC\".				?highcriticality a imonitor:CriticalityHigh.				?service imonitor:hasCriticality ?highcriticality.				?service imonitor:affects ?customers.}");

		assertThat(result, is(true));
	}
}
