package de.neusta.imonitor.rulewebservice;

/*
 * #%L
 * rulewebservice
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import java.io.File;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.List;

import javax.annotation.PostConstruct;

import org.apache.log4j.PropertyConfigurator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.neusta.imonitor.knowledgeserver.webservice.RuleDTO;
import de.neusta.imonitor.ontology.Ontology;
import de.neusta.imonitor.rulewebservice.services.OntologyInitService;
import de.neusta.imonitor.rulewebservice.services.RuleWebService;

/**
 * Use the the rule web service api for all interactions with the rule knowledge
 * service. You may want to set the custom webservice url by the the {@link setWebserviceUrl} function before
 * accessing the webservice the first time. 
 *
 */
@Service
public class RuleWebServiceApi {
	
	@Autowired
	private Ontology ontology;
	@Autowired
	private RuleWebService ruleWebService;
	@Autowired
	private OntologyInitService ontologyInitService;

	/**
	 * Optionally to configure the logger for the default logging.
	 */
	public void configureLogging() {
		File logConfigFile = new File("../configs/log4j.properties");
		if (logConfigFile.exists()) {
			System.out.println("Taking log config from '../configs/log4j.properties'");
			PropertyConfigurator.configure("../configs/log4j.properties");
		} else {
			InputStream inputStream = getClass().getResourceAsStream("/log4j.properties");
			if (inputStream == null) {
				System.out.println("Could not find log4j config file. Logging may not work.");
			} else {
				PropertyConfigurator.configure(inputStream);
			}
		}
	}

	/**
	 * Initializes the webservice by loading the top level ontology.
	 * 
	 * @throws Exception
	 */
	@PostConstruct
	public void init() throws Exception {
		this.ontologyInitService.loadAndInitOntology();
	}
	
	/**
	 * Sets the url to the webservice. Use this before calling the webservice the first time.
	 * @param url the url for the webservice.
	 */
	public void setWebserviceUrl(URL url) {
		ruleWebService.setUrl(url);
	}

	/**
	 * Gets all rules from the web service.
	 * 
	 * @return the rules fetched from the webservice.
	 * @throws MalformedURLException
	 */
	public List<RuleDTO> getRules() throws MalformedURLException {
		return ruleWebService.getRules();
	}

	/**
	 * Commits a rule from the local database to the knowledge service by using
	 * the webservice.
	 * 
	 * @param ruleDto
	 *            the rule to commit.
	 *            @return true on success.
	 * @throws Exception
	 *             exception.
	 */
	public boolean commitRule(RuleDTO ruleDto) throws Exception {
		return ruleWebService.commitRule(ruleDto);
	}
	
	public RuleDTO getRule(long id) throws MalformedURLException {
		return ruleWebService.getRule(id);
	}
}
