package de.neusta.imonitor.rulewebservice.factory;

/*
 * #%L
 * rulewebservice
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import java.net.MalformedURLException;
import java.net.URL;

import javax.xml.namespace.QName;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.neusta.imonitor.knowledgeserver.webservice.RuleService;
import de.neusta.imonitor.knowledgeserver.webservice.RuleService_Service;
import de.neusta.imonitor.rulewebservice.configs.WebServiceConfig;

@Service
public class RuleWebServiceFactory {

	private Logger LOGGER = LoggerFactory.getLogger(RuleWebServiceFactory.class);

	@Autowired
	private WebServiceConfig webServiceConfig;
	

	/**
	 * Creates a new rule service by using the webservice config file.
	 * @return an instance of the rule webservice.
	 * @throws MalformedURLException exception is the url is in a wrong format.
	 */
	public RuleService create() throws MalformedURLException {
		URL url;
		LOGGER.info(String.format("Using webservice url from config file: '%s'", webServiceConfig.getWsdlUrl()));
		url = new URL(webServiceConfig.getWsdlUrl());
		LOGGER.info(String.format("Connecting to rule service with url: '%s'", url.getPath()));
		javax.xml.ws.Service ruleService = RuleService_Service.create(url, new QName("http://webservice.knowledgeserver.imonitor.neusta.de/", "RuleService"));
		return ruleService.getPort(RuleService.class);
	}

	/**
	 * Creates a new rule service by using the given webservice url.
	 * @param url the url to use for the webservice.
	 * @return an instance of the webservice.
	 */
	public RuleService create(URL url)  {
		LOGGER.info(String.format("Connecting to rule service with url: '%s'", url.getPath()));
		javax.xml.ws.Service ruleService = RuleService_Service.create(url, new QName("http://webservice.knowledgeserver.imonitor.neusta.de/", "RuleService"));
		return ruleService.getPort(RuleService.class);
	}

}
