package de.neusta.imonitor.rulewebservice.services;

/*
 * #%L
 * rulewebservice
 * %%
 * Copyright (C) 2013 - 2015 neusta
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * #L%
 */


import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.List;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.runners.MockitoJUnitRunner;

import de.neusta.imonitor.knowledgeserver.webservice.RuleDTO;
import de.neusta.imonitor.knowledgeserver.webservice.RuleService;
import de.neusta.imonitor.ruleparser.independencychecker.IndependentRuleChecker;
import de.neusta.imonitor.ruleparser.rules.Rule;
import de.neusta.imonitor.ruleparser.rules.RuleParser;
import de.neusta.imonitor.rulewebservice.factory.RuleWebServiceFactory;
import de.neusta.imonitor.rulewebservice.services.RuleWebService;

@RunWith(MockitoJUnitRunner.class)
public class RuleWebServiceTest {

	@InjectMocks
	private RuleWebService rulesWebService;
	@Mock
	private RuleWebServiceFactory ruleWebServiceFactory;
	@Mock
	private RuleService ruleService;
	@Mock
	private IndependentRuleChecker independentRuleChecker;
	@Mock
	private RuleParser ruleParser;


	@Before
	public void setup() throws MalformedURLException {
		Mockito.when(ruleWebServiceFactory.create()).thenReturn(ruleService);
	}


	@Test
	public void testGetRules() throws Exception {
		List<RuleDTO> rules = new ArrayList<RuleDTO>();
		Mockito.when(ruleService.getRules()).thenReturn(rules);

		List<RuleDTO> result = rulesWebService.getRules();

		assertEquals(rules, result);
	}

	@Test
	public void testCommitRule() throws Exception {
		RuleDTO ruleDto = new RuleDTO();
		ruleDto.setXml("xml");
		Mockito.when(ruleService.commitRule(ruleDto)).thenReturn(true);
		Rule rule = new Rule();
		Mockito.when(ruleParser.parseRule(ruleDto.getXml())).thenReturn(rule);
		Mockito.when(independentRuleChecker.isIndependent(rule)).thenReturn(true);

		boolean result = rulesWebService.commitRule(ruleDto);

		assertTrue(result);
		Mockito.verify(independentRuleChecker).isIndependent(rule);
		Mockito.verify(ruleService).commitRule(ruleDto);
	}
	
	@Test
	public void testCommitRuleNotIndependent() throws Exception {
		RuleDTO ruleDto = new RuleDTO();
		ruleDto.setXml("xml");
		Mockito.when(ruleService.commitRule(ruleDto)).thenReturn(true);
		Rule rule = new Rule();
		Mockito.when(ruleParser.parseRule(ruleDto.getXml())).thenReturn(rule);
		Mockito.when(independentRuleChecker.isIndependent(rule)).thenReturn(false);

		boolean result = rulesWebService.commitRule(ruleDto);

		assertFalse(result);
		Mockito.verify(ruleService, Mockito.times(0)).commitRule(ruleDto);
	}


	@Test
		public void testGetRuleShouldReturnDto() throws Exception {
			RuleDTO rule = new RuleDTO();
			
			Mockito.when(ruleService.getRule(1l)).thenReturn(rule);
			
			RuleDTO result = rulesWebService.getRule(1l);
					
			assertNotNull(result);
		}
}