package de.neusta.imonitor.correlation;

import java.util.List;

import junit.framework.Assert;

import org.junit.Test;

import com.hp.hpl.jena.ontology.Individual;
import com.hp.hpl.jena.ontology.OntClass;
import com.hp.hpl.jena.query.ResultSet;

import de.neusta.imonitor.ontology.Ontology;

public class OntologyTests {

	@Test
	public void testOntologyCreateInstance(){
		Ontology ontology = TestHelper.getTestOntology();
		Assert.assertNotNull(ontology);
	}

	@Test
	public void testOntologyAsk(){
		Ontology ontology = TestHelper.getTestOntology();
		boolean result = ontology.ask("PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>" +
				"PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>" +
				"PREFIX imonitor: <http://www.imonitor-project.com/ontology#>" +
				"ASK  { imonitor:WebServer rdfs:subClassOf imonitor:Server; }");
		Assert.assertTrue(result);
	}
	
	@Test
	public void testOntologyCreateIndividual(){
		Ontology ontology = TestHelper.getTestOntology();
		String individualUri = "imonitor:testIndividual";
		Individual individual = ontology.createIndividual(individualUri, "imonitor:WebServer");
		Assert.assertNotNull(individual);
		Assert.assertEquals(individualUri, individual.getURI());
	}
	
	@Test
	public void testOntologySelectAllServers(){
		Ontology ontology = TestHelper.getTestOntology();
		ResultSet result = ontology.select("PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>" +
				"PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>" +
				"PREFIX imonitor: <http://www.imonitor-project.com/ontology#>" +
				"SELECT ?x { ?x rdfs:subClassOf imonitor:Server; }");
		List<String> variables = result.getResultVars();
		Assert.assertEquals(1, variables.size());
		Assert.assertEquals("x", variables.get(0));
		Assert.assertEquals(true, result.hasNext());
	}

	
	@Test
	public void testOntologyGetConcept(){
		Ontology ontology = TestHelper.getTestOntology();
		OntClass concept = ontology.getConcept("http://www.imonitor-project.com/ontology#Server");
		Assert.assertNotNull(concept);
		Assert.assertEquals("Server",concept.getLocalName());
	}

	@Test
	public void testOntologyGetIndividual(){
		Ontology ontology = TestHelper.getTestOntology();
		Individual individual = ontology.getIndividual("http://www.imonitor-project.com/ontology#EmptyRecycleBin");
		Assert.assertNotNull(individual);
		Assert.assertEquals("EmptyRecycleBin",individual.getLocalName());
	}

	
	@Test
	public void testGetSparqlPrefixes(){
		Ontology ontology = TestHelper.getTestOntology();
		Assert.assertNotNull(ontology.getSparqlPrefixes());
	}
}
