package de.neusta.imonitor.correlation;

import junit.framework.Assert;

import org.junit.Test;

import de.neusta.imonitor.ontology.Variable;
import de.neusta.imonitor.ontology.VariableList;

public class VariableListTests {

	@Test
	public void testVariableListCreateVariable(){
		VariableList variableList = new VariableList();
		try {
			String varName = "testVar";
			String varValue = "testVal";
			Variable variable = variableList.createVariable(varName, varValue);
			Assert.assertNotNull(variable);
			Assert.assertEquals(varName, variable.getName());
			Assert.assertEquals(varValue, variable.getValue());
			
		} catch (Exception e) {
			Assert.fail(e.getMessage());
		}
	}
	
	@Test
	public void testVariableListCreateAlreadyExistingVariableShouldThrowException(){
		VariableList variableList = new VariableList();
		boolean exception = false;
		try {
			String varName = "testVar";
			String varValue = "testVal";
			variableList.createVariable(varName, varValue);
			variableList.createVariable(varName, varValue);
		} catch (Exception e) {
			exception = true;
		}
		Assert.assertTrue("Exception expected.", exception);
	}
	
	@Test
	public void testVariableListClear(){
		VariableList variableList = new VariableList();
		String varName = "testVar";
		String varValue = "testVal";
		try {
			variableList.createVariable(varName, varValue);
		} catch (Exception e) {
			Assert.fail();
		}
		variableList.clearVariables();
		Variable result = variableList.getVariable(varName);
		Assert.assertNull("Expecting no results.", result);
	}
	
	@Test
	public void testVariableListGetVariable(){
		VariableList variableList = new VariableList();
		String varName = "testVar";
		String varValue = "testVal";
		try {
			variableList.createVariable(varName, varValue);
		} catch (Exception e) {
			Assert.fail();
		}
		Variable result = variableList.getVariable(varName);
		Assert.assertNotNull(result);
		Assert.assertEquals(varName, result.getName());
		Assert.assertEquals(varValue, result.getValue());
	}
	
	@Test
	public void testVariableListGetVariableShouldNotReturnUnknownVariable(){
		VariableList variableList = new VariableList();
		Variable result = variableList.getVariable("testVar");
		Assert.assertNull(result);
	}
}
